# SPDX-FileCopyrightText: 2024 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later

from argparse import Namespace

from gvm.protocols.gmp import Gmp
from gvmtools.helper import Table


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=unused-argument

    response_xml = gmp.get_users(filter_string="rows=-1")
    users_xml = response_xml.xpath("user")

    heading = ["#", "Name", "Id", "Role", "Groups"]

    rows = []
    numberRows = 0

    print("Listing users.\n")

    for user in users_xml:
        # Count number of reports
        numberRows = numberRows + 1
        # Cast/convert to text to show in list
        rowNumber = str(numberRows)

        name = "".join(user.xpath("name/text()"))
        user_id = user.get("id")
        user_role = "".join(user.xpath("role/name/text()"))
        user_groups = "".join(user.xpath("groups/group/name/text()"))

        rows.append([rowNumber, name, user_id, user_role, user_groups])

    print(Table(heading=heading, rows=rows))


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2017-2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import sys
from argparse import Namespace

from gvm.protocols.gmp import Gmp
from gvmtools.helper import create_xml_tree
from lxml.etree import Element


def check_args(gmp: Gmp, args: Namespace) -> None:
    len_args = len(args.script) - 1
    if len_args != 1:
        message = """
        This script pulls schedule data from an xml document and feeds it to \
    a desired GSM
        One parameter after the script name is required.

        1. <xml_doc>  -- .xml file containing schedules

        Example:
            $ gvm-script --gmp-username name --gmp-password pass \
    ssh --hostname <gsm> scripts/send-schedules.gmp.py example_file.xml

        """
        print(message)
        sys.exit()
    major, minor = gmp.get_protocol_version()
    if major < 21 and minor < 5:
        print(f"This script requires GMP version {major}.{minor}")
        sys.exit()


def parse_send_xml_tree(gmp: Gmp, xml_tree: Element) -> None:
    for schedule in xml_tree.xpath("schedule"):
        name = schedule.find("name").text

        comment = schedule.find("comment").text
        if comment is None:
            comment = ""

        ical = schedule.find("icalendar").text

        timezone = schedule.find("timezone").text

        gmp.create_schedule(
            name=name, comment=comment, timezone=timezone, icalendar=ical
        )


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable

    check_args(gmp=gmp, args=args)

    xml_doc = args.script[1]

    print("\nSending schedules...")

    xml_tree = create_xml_tree(xml_doc)
    parse_send_xml_tree(gmp, xml_tree)

    print("\n  Schedule(s) created!\n")


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2024 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

# Run with gvm-script --gmp-username admin-user --gmp-password password socket create-schedules-from-csv.gmp.py schedules.csv

import csv
import sys
import time
from argparse import ArgumentParser, Namespace, RawTextHelpFormatter
from pathlib import Path

from gvm.errors import GvmResponseError
from gvm.protocols.gmp import Gmp
from gvmtools.helper import error_and_exit

HELP_TEXT = (
    "This script pulls schedule information "
    "from a csv file and creates a schedule for each row. \n"
    "use the same schedule names when creating tasks! \n\n"
    "csv file may contain Name of schedule, Timezone, Icalendar entry \n"
    "Use example schedules.csv as a template \n\n"
    "It should be rather self explanatory."
)


def check_args(args):
    len_args = len(args.script) - 1
    if len_args != 2:
        message = """
        This script pulls schedules from a csv file and creates a \
schedule for each row in the csv file.
        One parameter after the script name is required.

        1. <schedules_csvfile>  -- csv file containing names and secrets required for scan schedules

        Example:
            $ gvm-script --gmp-username name --gmp-password pass \
ssh --hostname <gsm> scripts/create_schedules_from_csv.gmp.py \
<schedules-csvfile>
        """
        print(message)
        sys.exit()


def parse_args(args: Namespace) -> Namespace:  # pylint: disable=unused-argument
    """Parsing args ..."""

    parser = ArgumentParser(
        prefix_chars="+",
        add_help=False,
        formatter_class=RawTextHelpFormatter,
        description=HELP_TEXT,
    )

    parser.add_argument(
        "+h",
        "++help",
        action="help",
        help="Show this help message and exit.",
    )

    parser.add_argument(
        "sched_file",
        type=str,
        help=("CSV File containing schedules"),
    )
    script_args, _ = parser.parse_known_args(args)
    return script_args


def schedule_id(
    gmp: Gmp,
    schedule_name: str,
):
    response_xml = gmp.get_schedules(
        filter_string="rows=-1, name=" + schedule_name
    )
    schedules_xml = response_xml.xpath("schedule")
    schedule_id = ""

    for schedule in schedules_xml:
        schedule_id = schedule.get("id")
    return schedule_id


def create_schedules(
    gmp: Gmp,
    sched_file: Path,
):
    try:
        numberschedules = 0
        with open(sched_file, encoding="utf-8") as csvFile:
            content = csv.reader(csvFile, delimiter=",")  # read the data
            for row in content:  # loop through each row
                if len(row) == 0:
                    continue
                sched_name = row[0]
                sched_tz = row[1]
                sched_ical = row[2]
                comment = f"Created: {time.strftime('%Y/%m/%d-%H:%M:%S')}"
                try:
                    if schedule_id(gmp, sched_name):
                        print(f"Schedule: {sched_name} exist, not creating...")
                        continue
                    print("Creating schedule: " + sched_name)
                    gmp.create_schedule(
                        name=sched_name,
                        timezone=sched_tz,
                        icalendar=sched_ical,
                        comment=comment,
                    )
                    numberschedules = numberschedules + 1
                except GvmResponseError as gvmerr:
                    print(f"{gvmerr=}, name: {sched_name}")
                    pass
        csvFile.close()  # close the csv file

    except IOError as e:
        error_and_exit(f"Failed to read sched_file: {str(e)} (exit)")

    if len(row) == 0:
        error_and_exit("schedules file is empty (exit)")

    return numberschedules


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable
    if args.script:
        args = args.script[1:]

    parsed_args = parse_args(args=args)

    print("Creating schedules.\n")

    numberschedules = create_schedules(
        gmp,
        parsed_args.sched_file,
    )

    numberschedules = str(numberschedules)
    print("    [" + numberschedules + "] schedule(s) created!\n")


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2017-2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import sys
import time
from argparse import ArgumentParser, Namespace, RawTextHelpFormatter
from pathlib import Path
from typing import List

from gvm.protocols.gmp import Gmp
from gvmtools.helper import error_and_exit

HELP_TEXT = (
    "This script pulls hostnames from a text "
    "file and creates a target for each."
)


def check_args(args):
    len_args = len(args.script) - 1
    if len_args != 2:
        message = """
        This script pulls hostnames from a text file and creates a target \
for each.
        One parameter after the script name is required.

        1. <hostname>        -- IP of the GVM host
        2. <hosts_textfile>  -- text file containing hostnames

        Example:
            $ gvm-script --gmp-username name --gmp-password pass \
ssh --hostname <gsm> scripts/create_targets_from_host_list.gmp \
<hostname> <hosts_textfile>
        """
        print(message)
        sys.exit()


def parse_args(args: Namespace) -> Namespace:  # pylint: disable=unused-argument
    """Parsing args ..."""

    parser = ArgumentParser(
        prefix_chars="+",
        add_help=False,
        formatter_class=RawTextHelpFormatter,
        description=HELP_TEXT,
    )

    parser.add_argument(
        "+h",
        "++help",
        action="help",
        help="Show this help message and exit.",
    )

    parser.add_argument(
        "hostname",
        type=str,
        help="Host name to create targets for.",
    )

    parser.add_argument(
        "hosts_file",
        type=str,
        help=("File containing host names / IPs"),
    )

    ports = parser.add_mutually_exclusive_group()
    ports.add_argument(
        "+pl",
        "++port-list-id",
        type=str,
        dest="port_list_id",
        help="UUID of existing port list.",
    )
    ports.add_argument(
        "+pr",
        "++port-range",
        dest="port_range",
        type=str,
        help=(
            "Port range to create port list from, e.g. "
            "T:1-1234 for ports 1-1234/TCP"
        ),
    )

    ports.set_defaults(
        port_list_id="4a4717fe-57d2-11e1-9a26-406186ea4fc5"
    )  # All IANA assigned TCP and UDP
    script_args, _ = parser.parse_known_args(args)
    return script_args


def load_host_list(host_file):
    try:
        with open(host_file, encoding="utf-8") as f:
            content = f.readlines()
        host_list = [x.strip() for x in content]
        host_list = list(filter(None, host_list))
    except IOError as e:
        error_and_exit(f"Failed to read host_file: {str(e)} (exit)")

    if len(host_list) == 0:
        error_and_exit("Host file is empty (exit)")

    return host_list


def send_targets(
    gmp: Gmp,
    host_name: str,
    host_file: Path,
    host_list: List[str],
    port_list_id: str,
):
    print(f"\nSending targets from {host_file} to {host_name}...")

    for host in host_list:
        name = f"Target for {host}"
        comment = f"Created: {time.strftime('%Y/%m/%d-%H:%M:%S')}"
        hosts = [host]

        gmp.create_target(
            name=name, comment=comment, hosts=hosts, port_list_id=port_list_id
        )


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable
    if args.script:
        args = args.script[1:]

    parsed_args = parse_args(args=args)

    hosts_list = load_host_list(parsed_args.hosts_file)

    if parsed_args.port_range:
        print(parsed_args.port_range)
        resp = gmp.create_port_list(
            name=f"Port list for target {parsed_args.hostname}",
            port_range=parsed_args.port_range,
            comment="Port List created by gvm-script",
        )
        port_list_id = resp.xpath("//@id")[0]
        print(f"Port list {port_list_id} created!\n")
    else:
        port_list_id = parsed_args.port_list_id
    send_targets(
        gmp,
        parsed_args.hostname,
        parsed_args.hosts_file,
        hosts_list,
        port_list_id,
    )

    print("\n  Target(s) created!\n")


if __name__ == "__gmp__":
    main(gmp, args)



# -*- coding: utf-8 -*-
# SPDX-FileCopyrightText: 2020-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from importlib.util import module_from_spec, spec_from_file_location
from pathlib import Path
from typing import Union
from unittest.mock import MagicMock, create_autospec

from gvm.protocols.latest import Gmp
from lxml import etree


def load_script(path: Union[str, Path], script_name: str):
    """loading a script for a test case"""
    spec = spec_from_file_location(
        script_name, f"{str(path)}/{script_name}.gmp.py"
    )
    script = module_from_spec(spec)
    spec.loader.exec_module(script)

    return script


class GmpMockFactory:
    def __init__(self, *args, **kwargs):  # pylint: disable=unused-argument
        gmp_protocol_mock = create_autospec(Gmp)

        self.gmp_protocol = gmp_protocol_mock
        self.gmp = MagicMock()
        self.gmp.is_authenticated = MagicMock(return_value=True)
        self.gmp_protocol.types = MagicMock()
        self.gmp.__enter__.return_value = gmp_protocol_mock

    def __call__(self, *args, **kwargs):
        return self.gmp

    def mock_response(self, request_name: str, content: str):
        func = getattr(self.gmp_protocol, request_name)
        func.return_value = etree.fromstring(content)

    def mock_responses(self, request_name: str, content: str):
        func = getattr(self.gmp_protocol, request_name)
        func.side_effect = [etree.fromstring(c) for c in content]



# SPDX-FileCopyrightText: 2024 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

# Loosely based on the create-targets-from-host-list
#
# Run with gvm-script --gmp-username admin-user --gmp-password password socket create-tasks-from-csv.gmp.py hostname-server tasks.csv

# Note: for some weird reason theres a space in front of the default scan config " Full and fast"
# Examples of all defaults in tasks.csv

import csv
import sys
import time
from argparse import ArgumentParser, Namespace, RawTextHelpFormatter
from pathlib import Path

from gvm.errors import GvmResponseError
from gvm.protocols.gmp import Gmp
from gvmtools.helper import error_and_exit

HELP_TEXT = (
    "This script pulls taskname, scanner, & scan config "
    "from a csv file and creates a task for each row. \n\n"
    "csv file to contain name of task, target name, scanner name, and scan config name  \n"
    "name,target,scanner,scan config"
)


def check_args(args):
    len_args = len(args.script) - 1
    if len_args != 1:
        message = """
        This script pulls taskinformation from a csv file and creates a task \
for each row.
        One parameter after the script name is required.

        1. <tasks_csvfile>  -- text file containing taskname, target name, scanner name, and scan config name

        Example:
            $ gvm-script --gmp-username name --gmp-password pass \
ssh --hostname <gsm> scripts/create_tasks_from_csv.gmp <tasks_csvfile>
        """
        print(message)
        sys.exit()


def parse_args(args: Namespace) -> Namespace:  # pylint: disable=unused-argument
    """Parsing args ..."""

    parser = ArgumentParser(
        prefix_chars="+",
        add_help=False,
        formatter_class=RawTextHelpFormatter,
        description=HELP_TEXT,
    )

    parser.add_argument(
        "+h",
        "++help",
        action="help",
        help="Show this help message and exit.",
    )

    parser.add_argument(
        "tasks_csv_file",
        type=str,
        help=("File containing host names / IPs"),
    )

    ports = parser.add_mutually_exclusive_group()
    ports.add_argument(
        "+pl",
        "++port-list-id",
        type=str,
        dest="port_list_id",
        help="UUID of existing port list.",
    )
    ports.add_argument(
        "+pr",
        "++port-range",
        dest="port_range",
        type=str,
        help=(
            "Port range to create port list from, e.g. "
            "T:1-1234 for ports 1-1234/TCP"
        ),
    )

    ports.set_defaults(
        port_list_id="730ef368-57e2-11e1-a90f-406186ea4fc5"
    )  # All IANA assigned TCP and Top 100 UDP, use the list-portlists.gmp.py script to get the UUID for all.
    # Defaults are
    # # | Name                          | ID
    # - | ----------------------------- | ------------------------------------
    # 1 | All IANA assigned TCP         | 33d0cd82-57c6-11e1-8ed1-406186ea4fc5
    # 2 | All IANA assigned TCP and UDP | 4a4717fe-57d2-11e1-9a26-406186ea4fc5
    # 3 | All TCP and Nmap top 100 UDP  | 730ef368-57e2-11e1-a90f-406186ea4fc5

    script_args, _ = parser.parse_known_args(args)
    return script_args


def config_id(
    gmp: Gmp,
    config_name: str,
):
    response_xml = gmp.get_scan_configs(
        filter_string="rows=-1, name= " + config_name
    )
    scan_configs_xml = response_xml.xpath("config")
    config_id = ""

    for scan_config in scan_configs_xml:
        config_id = scan_config.get("id")
    return config_id


def alert_id(
    gmp: Gmp,
    alert_name: str,
):
    response_xml = gmp.get_alerts(filter_string="rows=-1, name=" + alert_name)
    alerts_xml = response_xml.xpath("alert")
    alert_id = ""

    for alert in alerts_xml:
        alert_id = alert.get("id")
    return alert_id


def target_id(
    gmp: Gmp,
    target_name: str,
):
    response_xml = gmp.get_targets(filter_string="rows=-1, name=" + target_name)
    targets_xml = response_xml.xpath("target")
    target_id = ""

    for target in targets_xml:
        target_id = target.get("id")
    return target_id


def scanner_id(
    gmp: Gmp,
    scanner_name: str,
):
    response_xml = gmp.get_scanners(
        filter_string="rows=-1, name=" + scanner_name
    )
    scanners_xml = response_xml.xpath("scanner")
    scanner_id = ""

    for scanner in scanners_xml:
        scanner_id = scanner.get("id")
    return scanner_id


def schedule_id(
    gmp: Gmp,
    schedule_name: str,
):
    response_xml = gmp.get_schedules(
        filter_string="rows=-1, name=" + schedule_name
    )
    schedules_xml = response_xml.xpath("schedule")
    schedule_id = ""

    for schedule in schedules_xml:
        schedule_id = schedule.get("id")
    return schedule_id


def task_id(
    gmp: Gmp,
    taskName: str,
):
    response_xml = gmp.get_tasks(filter_string="rows=-1, name=" + taskName)
    tasks_xml = response_xml.xpath("task")
    task_id = ""

    for task in tasks_xml:
        task_id = task.get("id")
    return task_id


def create_tasks(
    gmp: Gmp,
    task_csv_file: Path,
    port_list_id: str,
):
    try:
        numberTasks = 0
        with open(task_csv_file, encoding="utf-8") as csvFile:
            content = csv.reader(csvFile, delimiter=",")  # read the data
            for row in content:  # loop through each row
                if len(row) == 0:
                    continue
                name = row[0]
                targetId = target_id(gmp, row[1])
                scannerId = scanner_id(gmp, row[2])
                alterable = "True"
                configId = config_id(gmp, row[3])
                scheduleId = schedule_id(gmp, row[4])

                newOrder = row[5].upper()
                if newOrder == "RANDOM":
                    order = gmp.types.HostsOrdering.RANDOM
                elif newOrder == "SEQUENTIAL":
                    order = gmp.types.HostsOrdering.SEQUENTIAL
                elif newOrder == "REVERSE":
                    order = gmp.types.HostsOrdering.REVERSE
                else:
                    order = gmp.types.HostsOrdering.RANDOM

                alerts = []
                if len(row[6]) > 1:
                    alert = alert_id(gmp, row[6])
                    alerts.append(alert)
                if len(row[7]) > 1:
                    alert = alert_id(gmp, row[7])
                    alerts.append(alert)
                if len(row[8]) > 1:
                    alert = alert_id(gmp, row[8])
                    alerts.append(alert)
                if len(row[9]) > 1:
                    alert = alert_id(gmp, row[9])
                    alerts.append(alert)
                if len(row[10]) > 1:
                    alert = alert_id(gmp, row[10])
                    alerts.append(alert)

                scanOrder = order  # Use SEQUENTIAL, REVERSE, or RANDOM
                comment = f"Created: {time.strftime('%Y/%m/%d-%H:%M:%S')}"

                try:
                    if task_id(gmp, name):
                        print(f"Task: {name} exist already")
                        continue
                    print("Creating task: " + name)
                    gmp.create_task(
                        name=name,
                        comment=comment,
                        config_id=configId,
                        target_id=targetId,
                        hosts_ordering=scanOrder,
                        scanner_id=scannerId,
                        alterable=alterable,
                        schedule_id=scheduleId,
                        alert_ids=alerts,
                    )
                    numberTasks = numberTasks + 1
                except GvmResponseError as gvmerr:
                    print(f"{gvmerr=}, name: {name}")
                    pass
        csvFile.close()  # close the csv file

    except IOError as e:
        error_and_exit(f"Failed to read task_csv_file: {str(e)} (exit)")

    if len(row) == 0:
        error_and_exit("Host file is empty (exit)")

    return numberTasks


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable
    if args.script:
        args = args.script[1:]

    parsed_args = parse_args(args=args)
    # port_list_id="4a4717fe-57d2-11e1-9a26-406186ea4fc5"

    print("Creating tasks\n")

    numberTasks = create_tasks(
        gmp, parsed_args.tasks_csv_file, parsed_args.port_list_id
    )

    numberTasks = str(numberTasks)
    print("    [" + numberTasks + "] task(s) created!\n")


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2024 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

# Loosely based on the create-targets-from-host-list.gmp.py
# As provided by Greenbone in the gvm-tools repo
#
# Run with gvm-script --gmp-username admin-user --gmp-password password socket create-credentials-from-csv.gmp.py credentials.csv

import csv
import sys
import time
from argparse import ArgumentParser, Namespace, RawTextHelpFormatter
from pathlib import Path

from gvm.errors import GvmResponseError
from gvm.protocols.gmp import Gmp
from gvmtools.helper import error_and_exit

HELP_TEXT = (
    "This script pulls Credential information "
    "from a csv file and creates a credential for each row. \n"
    "use the same credential names when creating targets! \n\n"
    "csv file may contain Name of target, Login, password, and ssh-key \n"
    "Name,Type,Login,Password,ssh-key \n\n"
    "Please note: SNMP and ESX not supported yet "
)


def check_args(args):
    len_args = len(args.script) - 1
    if len_args != 2:
        message = """
        This script pulls credentials from a csv file and creates a \
credential for each row in the csv file.
        One parameter after the script name is required.

        1. <credentials_csvfile>  -- csv file containing names and secrets required for scan credentials

        Example:
            $ gvm-script --gmp-username name --gmp-password pass \
ssh --hostname <gsm> scripts/create_credentials_from_csv.gmp.py \
<credentials-csvfile>
        """
        print(message)
        sys.exit()


def parse_args(args: Namespace) -> Namespace:  # pylint: disable=unused-argument
    """Parsing args ..."""

    parser = ArgumentParser(
        prefix_chars="+",
        add_help=False,
        formatter_class=RawTextHelpFormatter,
        description=HELP_TEXT,
    )

    parser.add_argument(
        "+h",
        "++help",
        action="help",
        help="Show this help message and exit.",
    )

    parser.add_argument(
        "cred_file",
        type=str,
        help=("CSV File containing credentials"),
    )
    script_args, _ = parser.parse_known_args(args)
    return script_args


def credential_id(
    gmp: Gmp,
    credName: str,
):
    response_xml = gmp.get_credentials(
        filter_string="rows=-1, name=" + credName
    )
    credentials_xml = response_xml.xpath("credential")
    cred_id = ""

    for credential in credentials_xml:
        cred_id = credential.get("id")
    return cred_id


def create_credentials(
    gmp: Gmp,
    cred_file: Path,
):
    try:
        numberCredentials = 0
        with open(cred_file, encoding="utf-8") as csvFile:
            content = csv.reader(csvFile, delimiter=",")  # read the data
            for row in content:  # loop through each row
                if len(row) == 0:
                    continue
                cred_name = row[0]
                cred_type = row[1]
                userName = row[2]
                userPW = row[3]
                comment = f"Created: {time.strftime('%Y/%m/%d-%H:%M:%S')}"

                if credential_id(gmp, cred_name):
                    print(f"Credential: {cred_name} exist, not creating...")
                    continue

                if cred_type == "UP":
                    try:
                        print("Creating credential: " + cred_name)
                        gmp.create_credential(
                            name=cred_name,
                            credential_type=gmp.types.CredentialType.USERNAME_PASSWORD,
                            login=userName,
                            password=userPW,
                            comment=comment,
                        )
                        numberCredentials = numberCredentials + 1
                    except GvmResponseError as gvmerr:
                        print(f"{gvmerr=}, name: {cred_name}")
                        pass
                elif cred_type == "SSH":
                    with open(row[4]) as key_file:
                        key = key_file.read()

                    try:
                        print("Creating credential: " + cred_name)
                        gmp.create_credential(
                            name=cred_name,
                            credential_type=gmp.types.CredentialType.USERNAME_SSH_KEY,
                            login=userName,
                            key_phrase=userPW,
                            private_key=key,
                            comment=comment,
                        )
                        numberCredentials = numberCredentials + 1
                    except GvmResponseError as gvmerr:
                        print(f"{gvmerr=}, name: {cred_name}")
                        pass
                elif cred_type == "SNMP":
                    # Unfinished, copy of UP for now
                    try:
                        print("Creating credential: " + cred_name)
                        gmp.create_credential(
                            name=cred_name,
                            credential_type=gmp.types.CredentialType.USERNAME_SSH_KEY,
                            login=userName,
                            key_phrase=userPW,
                            private_key=key,
                            comment=comment,
                        )
                        numberCredentials = numberCredentials + 1
                    except GvmResponseError as gvmerr:
                        print(f"{gvmerr=}, name: {cred_name}")
                        pass

                elif cred_type == "ESX":
                    # Unfinished, copy of UP for now
                    try:
                        print("Creating credential: " + cred_name)
                        gmp.create_credential(
                            name=cred_name,
                            credential_type=gmp.types.CredentialType.USERNAME_SSH_KEY,
                            login=userName,
                            key_phrase=userPW,
                            private_key=key,
                            comment=comment,
                        )
                        numberCredentials = numberCredentials + 1
                    except GvmResponseError as gvmerr:
                        print(f"{gvmerr=}, name: {cred_name}")
                        pass
        csvFile.close()  # close the csv file

    except IOError as e:
        error_and_exit(f"Failed to read cred_file: {str(e)} (exit)")

    if len(row) == 0:
        error_and_exit("Credentials file is empty (exit)")

    return numberCredentials


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable
    if args.script:
        args = args.script[1:]

    parsed_args = parse_args(args=args)

    print("Creating credentials.\n")

    numberCredentials = create_credentials(
        gmp,
        parsed_args.cred_file,
    )

    numberCredentials = str(numberCredentials)
    print("    [" + numberCredentials + "] Credential(s) created!\n")


if __name__ == "__gmp__":
    main(gmp, args)



# -*- coding: utf-8 -*-
# SPDX-FileCopyrightText: 2019-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys


class SuppressOutput:
    def __init__(self, *, suppress_stdout=False, suppress_stderr=False):
        self.suppress_stdout = suppress_stdout
        self.suppress_stderr = suppress_stderr
        self.original_stdout = None
        self.original_stderr = None

    def __enter__(self):
        self.devnull = open(os.devnull, "w", encoding="utf-8")

        # Suppress streams
        if self.suppress_stdout:
            self.original_stdout = sys.stdout
            sys.stdout = self.devnull

        if self.suppress_stderr:
            self.original_stderr = sys.stderr
            sys.stderr = self.devnull

    def __exit__(self, *args, **kwargs):
        # Restore streams
        if self.suppress_stdout:
            sys.stdout = self.original_stdout

        if self.suppress_stderr:
            sys.stderr = self.original_stderr

        self.devnull.close()



# -*- coding: utf-8 -*-
#
# Configuration file for the Sphinx documentation builder.
#
# This file does only contain a selection of the most common options. For a
# full list see the documentation:
# http://www.sphinx-doc.org/en/master/config

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
# pylint: disable=invalid-name,redefined-builtin,wrong-import-position

import os
import sys

sys.path.insert(0, os.path.abspath(".."))

import gvmtools

# -- Project information -----------------------------------------------------

project = "gvm-tools"
copyright = "2018-2024, Greenbone AG"
author = "Greenbone AG"

version = gvmtools.get_version()
release = version


# -- General configuration ---------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.githubpages",
    "sphinx.ext.napoleon",
    "myst_parser",
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
source_suffix = [".md", ".rst"]

# The master toctree document.
master_doc = "index"

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = "en"

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ["build", "dist", ".venv"]

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "furo"
html_title = project

html_favicon = "favicon.png"

html_css_files = ["custom.css"]
html_logo = "_static/logo.svg"

html_static_path = ["_static"]

repo_url = "https://github.com/greenbone/gvm-tools/"

html_theme_options = {
    "source_repository": repo_url,
    "source_branch": "main",
    "source_directory": "src/",
    "light_css_variables": {
        "color-content-foreground": "#4C4C4C",
        "color-foreground-primary": "4C4C4C",
        "color-foreground-secondary": "#7F7F7F",
        "color-code-background": "#333333",
        "color-code-foreground": "#E5E5E5",
        "color-admonition-title--note": "#11AB51",
        "admonition-font-size": "0.9rem",
        "color-background-primary": "#FFFFFF",
        "color-background-secondary": "#F3F3F3",
        "color-sidebar-background": "#F3F3F3",
    },
    "dark_css_variables": {
        "color-content-foreground": "#F3F3F3",
        "color-foreground-primary": "F3F3F3",
        "color-foreground-secondary": "#E5E5E5",
        "color-code-background": "#333333",
        "color-code-foreground": "#E5E5E5",
        "color-admonition-title--note": "#11AB51",
        "admonition-font-size": "0.9rem",
        "color-background-primary": "#171717",
        "color-background-secondary": "#4C4C4C",
        "color-sidebar-background": "#333333",
    },
    "footer_icons": [
        {
            "name": "GitHub",
            "url": repo_url,
            "html": """
                <svg stroke="currentColor" fill="currentColor" stroke-width="0" viewBox="0 0 16 16">
                    <path fill-rule="evenodd" d="M8 0C3.58 0 0 3.58 0 8c0 3.54 2.29 6.53 5.47 7.59.4.07.55-.17.55-.38 0-.19-.01-.82-.01-1.49-2.01.37-2.53-.49-2.69-.94-.09-.23-.48-.94-.82-1.13-.28-.15-.68-.52-.01-.53.63-.01 1.08.58 1.23.82.72 1.21 1.87.87 2.33.66.07-.52.28-.87.51-1.07-1.78-.2-3.64-.89-3.64-3.95 0-.87.31-1.59.82-2.15-.08-.2-.36-1.02.08-2.12 0 0 .67-.21 2.2.82.64-.18 1.32-.27 2-.27.68 0 1.36.09 2 .27 1.53-1.04 2.2-.82 2.2-.82.44 1.1.16 1.92.08 2.12.51.56.82 1.27.82 2.15 0 3.07-1.87 3.75-3.65 3.95.29.25.54.73.54 1.48 0 1.07-.01 1.93-.01 2.2 0 .21.15.46.55.38A8.013 8.013 0 0 0 16 8c0-4.42-3.58-8-8-8z"></path>
                </svg>
            """,
            "class": "",
        },
    ],
}

autodoc_typehints = "description"
autodoc_typehints_description_target = "documented"

autodoc_class_signature = "separated"

pygments_style = "zenburn"

# -- Options for HTMLHelp output ---------------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = "gvm-toolsdoc"

# -- Options for LaTeX output ------------------------------------------------
latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
    # 'papersize': 'letterpaper',
    # The font size ('10pt', '11pt' or '12pt').
    #
    # 'pointsize': '10pt',
    # Additional stuff for the LaTeX preamble.
    #
    # 'preamble': '',
    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
}
# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (
        master_doc,
        "gvm-tools.tex",
        "gvm-tools Documentation",
        "Greenbone AG",
        "manual",
    )
]


# -- Options for manual page output ------------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [(master_doc, "gvm-tools", "gvm-tools Documentation", [author], 1)]


# -- Options for Texinfo output ----------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (
        master_doc,
        "gvm-tools",
        "gvm-tools Documentation",
        author,
        "gvm-tools",
        "One line description of project.",
        "Miscellaneous",
    )
]


# -- Options for Epub output -------------------------------------------------

# Bibliographic Dublin Core info.
epub_title = project

# The unique identifier of the text. This can be a ISBN number
# or the project homepage.
#
# epub_identifier = ''

# A unique identification for the text.
#
# epub_uid = ''

# A list of files that should not be packed into the epub file.
epub_exclude_files = ["search.html"]


# -- Extension configuration -------------------------------------------------



# -*- coding: utf-8 -*-
# SPDX-FileCopyrightText: 2020-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import ipaddress
import sys
import unittest
import uuid
from io import BytesIO
from pathlib import Path
from unittest.mock import MagicMock, patch

from gvm.errors import GvmError
from lxml import etree

from gvmtools.helper import (
    Table,
    authenticate,
    create_xml_tree,
    do_not_run_as_root,
    error_and_exit,
    generate_id,
    generate_random_ips,
    generate_uuid,
    run_script,
    yes_or_no,
)

CWD = Path(__file__).absolute().parent


class TableTestCase(unittest.TestCase):
    def setUp(self):
        self.heading = ["ID", "Name", "Severity"]
        self.rows = [
            ["1", "foobar", "high"],
            ["2", "bla", "low"],
            ["3", "blub", "medium"],
        ]
        self.divider = " - "

        self.table = Table(
            heading=self.heading, rows=self.rows, divider=self.divider
        )

    def test_init_no_args(self):
        table = Table()

        self.assertListEqual(table.heading, [])
        self.assertListEqual(table.rows, [])
        self.assertEqual(table.divider, " | ")

    def test_init_with_args(self):
        self.assertListEqual(self.table.heading, self.heading)
        self.assertListEqual(self.table.rows, self.rows)
        self.assertEqual(self.table.divider, self.divider)

    def test_calculate_dimensions(self):
        expected_result = [
            len(self.heading[0]),
            len(self.rows[0][1]),
            len(self.heading[2]),
        ]

        column_sizes = (
            self.table._calculate_dimensions()  # pylint: disable=protected-access
        )

        self.assertEqual(column_sizes, expected_result)

    def test_create_column(self):
        column = "foobar"
        size = 20
        expected = "foobar              "

        result = self.table._create_column(  # pylint: disable=protected-access
            column, size
        )

        self.assertEqual(result, expected)

    def test_create_row(self):
        columns = ["foo", "bar", "blub"]
        expected = self.divider.join(columns)

        result = self.table._create_row(columns)  # pylint: disable=W0212

        self.assertEqual(result, expected)

    def test_str(self):
        expected = (
            "ID - Name   - Severity\n"
            + "-- - ------ - --------\n"
            + "1  - foobar - high    \n"
            + "2  - bla    - low     \n"
            + "3  - blub   - medium  "
        )

        self.assertEqual(str(self.table), expected)


class DoNotRunAsRootTestCase(unittest.TestCase):
    @patch("gvmtools.helper.os")
    def test_do_not_run_as_root_as_root(self, mock_os):
        mock_os.geteuid = MagicMock(spec="geteuid")
        mock_os.geteuid.return_value = 0

        self.assertRaises(RuntimeError, do_not_run_as_root)

    @patch("gvmtools.helper.os")
    def test_do_not_run_as_root_as_non_root(self, mock_os):
        mock_os.geteuid = MagicMock(spec="geteuid")
        mock_os.geteuid.return_value = 123

        self.assertIsNone(do_not_run_as_root())


class AuthenticateTestCase(unittest.TestCase):
    def test_authenticate_already_authenticated(self):
        mock_gmp = self.create_gmp_mock(True)

        self.assertIsNone(authenticate(mock_gmp))

    @patch("gvmtools.helper.input", return_value="foo")
    def test_authenticate_username_is_none(
        self, mock_input
    ):  # pylint: disable=unused-argument,line-too-long
        mock_gmp = self.create_gmp_mock(False)

        return_value = authenticate(mock_gmp, password="bar")

        self.assertTrue(isinstance(return_value, tuple))
        self.assertEqual(return_value[0], "foo")
        self.assertEqual(return_value[1], "bar")

    @patch("gvmtools.helper.getpass")
    def test_authenticate_password_is_none(self, mock_getpass):
        mock_gmp = self.create_gmp_mock(False)
        mock_getpass.getpass = MagicMock(return_value="SuperSecret123!")

        return_value = authenticate(mock_gmp, username="user")

        self.assertTrue(isinstance(return_value, tuple))
        self.assertEqual(return_value[0], "user")
        self.assertEqual(return_value[1], "SuperSecret123!")

    def test_authenticate(self):
        mock_gmp = self.create_gmp_mock(False)

        return_value = authenticate(
            mock_gmp, username="user", password="password"
        )

        self.assertTrue(isinstance(return_value, tuple))
        self.assertEqual(return_value[0], "user")
        self.assertEqual(return_value[1], "password")

    def test_authenticate_bad_credentials(self):
        mock_gmp = self.create_gmp_mock(False)

        def my_authenticate(username, password):
            raise GvmError("foo")

        mock_gmp.authenticate = my_authenticate

        self.assertRaises(GvmError, authenticate, mock_gmp, "user", "password")

    def create_gmp_mock(self, authenticated_return_value):
        mock_gmp = MagicMock()
        mock_gmp.is_authenticated = MagicMock(
            return_value=authenticated_return_value
        )
        return mock_gmp


class RunScriptTestCase(unittest.TestCase):
    @patch("gvmtools.helper.open")
    @patch("gvmtools.helper.exec")
    def test_run_script(self, mock_exec, mock_open):
        path = "foo"
        global_vars = ["OpenVAS", "is", "awesome"]
        mock_open().read.return_value = "file content"

        run_script(path, global_vars)

        mock_open.assert_called_with(path, "r", encoding="utf-8", newline="")
        mock_exec.assert_called_with("file content", global_vars)

    @patch("gvmtools.helper.open")
    @patch("gvmtools.helper.print")
    def test_run_script_file_not_found(self, mock_print, mock_open):
        def my_open(path, mode, newline, encoding):
            raise FileNotFoundError

        mock_open.side_effect = my_open

        path = "foo"
        global_vars = ["OpenVAS", "is", "awesome"]

        with self.assertRaises(SystemExit):
            run_script(path, global_vars)

        mock_print.assert_called_with(
            f"Script {path} does not exist", file=sys.stderr
        )


class ScriptUtilsTestCase(unittest.TestCase):
    @patch("builtins.input", lambda *args: "y")
    def test_yes(self):
        yes = yes_or_no("foo?")
        self.assertTrue(yes)

    @patch("builtins.input", lambda *args: "n")
    def test_no(self):
        no = yes_or_no("bar?")
        self.assertFalse(no)

    def test_error_and_exit(self):
        with self.assertRaises(SystemExit):
            error_and_exit("foo")

    def test_create_xml_tree(self):
        tree = create_xml_tree(BytesIO(b"<foo><baz/><bar>glurp</bar></foo>"))
        self.assertIsInstance(
            tree, etree._Element  # pylint: disable=protected-access
        )
        self.assertEqual(tree.tag, "foo")

    def test_create_xml_tree_invalid_file(self):
        target_xml_path = CWD / "invalid_file.xml"

        with self.assertRaises(SystemExit):
            with self.assertRaises(OSError):
                create_xml_tree(str(target_xml_path))

    def test_create_xml_tree_invalid_xml(self):
        with self.assertRaises(SystemExit):
            with self.assertRaises(etree.Error):
                create_xml_tree(BytesIO(b"<foo><baz/><bar>glurp<bar></foo>"))

    def test_generate_uuid(self):
        random_uuid = generate_uuid()
        try:
            uuid.UUID(random_uuid, version=4)
        except (ValueError, TypeError, AttributeError):
            self.fail("No valid UUID.")

    def test_generate_id(self):
        random_id = generate_id(size=1, chars="a")
        self.assertEqual(random_id, "a")

        random_id = generate_id(size=10)
        self.assertEqual(len(random_id), 10)
        self.assertTrue(random_id.isalnum())

    def test_generate_random_ips(self):
        random_ip = generate_random_ips(1)
        ip_addr = ipaddress.ip_address(random_ip[0])
        self.assertEqual(ip_addr.version, 4)
        self.assertEqual(str(ip_addr), random_ip[0])

        num = 10
        random_ips = generate_random_ips(num)
        self.assertEqual(len(random_ips), num)



# SPDX-FileCopyrightText: 2024 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later

from argparse import Namespace

from gvm.protocols.gmp import Gmp
from gvmtools.helper import Table


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=unused-argument

    response_xml = gmp.get_policies(filter_string="rows=-1")
    policies_xml = response_xml.xpath("config")

    heading = ["#", "Name", "Id", "NVT Count"]

    rows = []
    numberRows = 0

    print("Listing compliance policies.\n")

    for policy in policies_xml:
        # Count number of reports
        numberRows = numberRows + 1
        # Cast/convert to text to show in list
        rowNumber = str(numberRows)

        name = "".join(policy.xpath("name/text()"))
        policy_id = policy.get("id")
        policy_nvt = "".join(policy.xpath("nvt_count/text()"))

        rows.append([rowNumber, name, policy_id, policy_nvt])

    print(Table(heading=heading, rows=rows))


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2017-2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import sys
import time
from argparse import Namespace

from gvm.protocols.gmp import Gmp


def check_args(args):
    len_args = len(args.script) - 1

    if len_args != 1:
        message = """
        This script deletes overrides with a specific filter value

        <filter>  -- the parameter for the filter.

        Example:
            $ gvm-script --gmp-username name --gmp-password pass \
    ssh --hostname <gsm> scripts/delete-overrides-by-filter.gmp.py <filter>
        """
        print(message)
        sys.exit()


def delete_overrides(gmp, filter_value):
    filters = gmp.get_overrides(filter=filter_value)

    if not filters.xpath("override"):
        print(f"No overrides with filter: {filter_value}")

    for f_id in filters.xpath("override/@id"):
        print(f"Delete override: {f_id}", end="")
        res = gmp.delete_override(f_id)

        if "OK" in res.xpath("@status_text")[0]:
            print(" OK")
        else:
            print(" ERROR")

        time.sleep(60)


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable

    check_args(args)

    filter_value = args.script[1]

    delete_overrides(gmp, filter_value)


if __name__ == "__gmp__":
    main(gmp, args)



# -*- coding: utf-8 -*-
# SPDX-FileCopyrightText: 2020-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest
from argparse import Namespace
from pathlib import Path
from unittest.mock import patch

from lxml import etree

from . import GmpMockFactory, load_script

CWD = Path(__file__).absolute().parent


class SendTargetTestCase(unittest.TestCase):
    def setUp(self):
        self.send_targets = load_script(
            (CWD.parent.parent / "scripts"), "send-targets"
        )

    @patch("gvm.protocols.latest.Gmp", new_callable=GmpMockFactory)
    def test_sent_target(self, mock_gmp: GmpMockFactory):
        target_xml_path = CWD / "example_target.xml"
        target_xml_str = target_xml_path.read_text(encoding="utf-8")

        mock_gmp.mock_response(
            "get_credentials",
            '<get_credentials_response status="200" status_text="OK">'
            '<credential id="6da5b7de-92ad-4dd2-8610-d5711b9c5937">'
            "</credential>"
            '<credential id="7802648d-1a31-4f69-bb30-00766a1ae1e6">'
            "</credential>"
            '<credential id="70a63257-4923-4bf4-a9bb-dd8b710b2d80">'
            "</credential>"
            '<credential id="2bac0c76-795e-4742-b17a-808a0ec8e409">'
            "</credential>"
            "</get_credentials_response>",
        )
        mock_gmp.mock_response(
            "create_target",
            '<create_target_response status="201" status_text="OK,'
            'resource created" id="6c9f73f5-f14c-42bf-ab44-edb8d2493dbc"/>',
        )

        target = etree.XML(target_xml_str)

        self.send_targets.parse_send_xml_tree(mock_gmp.gmp_protocol, target)

    @patch("builtins.input", lambda *args: "n")
    @patch("gvm.protocols.latest.Gmp", new_callable=GmpMockFactory)
    def test_sent_target_no_credential(self, mock_gmp: GmpMockFactory):
        target_xml_path = CWD / "example_target.xml"
        target_xml_str = target_xml_path.read_text(encoding="utf-8")

        mock_gmp.mock_response(
            "get_credentials",
            '<get_credentials_response status="200" status_text="OK">'
            '<credential id="70a63257-4923-4bf4-a9bb-dd8b710b2d80">'
            "</credential>"
            '<credential id="2bac0c76-795e-4742-b17a-808a0ec8e409">'
            "</credential>"
            "</get_credentials_response>",
        )
        mock_gmp.mock_response(
            "create_target",
            '<create_target_response status="201" status_text="OK, '
            'resource created" id="6c9f73f5-f14c-42bf-ab44-edb8d2493dbc"/>',
        )

        target = etree.XML(target_xml_str)

        with self.assertRaises(SystemExit):
            self.send_targets.parse_send_xml_tree(mock_gmp.gmp_protocol, target)

    def test_args(self):
        args = Namespace(script=["foo"])
        with self.assertRaises(SystemExit):
            self.send_targets.check_args(args)

        args2 = Namespace(script=["foo", "bar", "baz"])

        with self.assertRaises(SystemExit):
            self.send_targets.check_args(args2)



# SPDX-FileCopyrightText: 2017-2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

# GMP script for gvm-cli to send emails with delta reports.

import base64
import datetime
import sched
import smtplib
import sys
import time
from argparse import Namespace
from email.mime.base import MIMEBase
from email.mime.multipart import MIMEMultipart
from email.utils import formatdate

from gvm.protocols.gmp import Gmp


def check_args(args: Namespace) -> None:
    len_args = len(args.script) - 1
    if len_args != 0:
        message = """
        This script, once started, will continuously send delta
        reports via email for selected tasks.

        Example for starting up the routine:
            $ gvm-script --gmp-username name --gmp-password pass ssh \
                --hostname <gsm> scripts/send-delta-emails.gmp.py

        The routine follows this procedure:

        Every <interval> minutes do:
        Get all tasks where the tag <task_tag> is attached.
        For each of these tasks get the finished reports:
            If less than 2 reports, continue with next task
            If latest report has tag "delta_alert_sent", continue with next task
            Create a CSV report from the delta of latest vs. previous report
            where filtered for only the new results.
            Send the CSV as an attachment to the configured email address.
        """
        print(message)
        sys.exit()


def execute_send_delta_emails(sc: sched.scheduler, **kwargs: dict) -> None:
    gmp = kwargs.get("gmp")
    task_tag = kwargs.get("task_tag")
    interval = kwargs.get("interval")
    email_subject = kwargs.get("email_subject")
    to_addresses = kwargs.get("to_addresses")
    from_address = kwargs.get("from_address")
    mta_address = kwargs.get("mta_address")
    mta_user = kwargs.get("mta_user")
    mta_port = kwargs.get("mta_port")
    mta_password = kwargs.get("mta_password")
    report_tag_name = kwargs.get("report_tag_name")

    print("Retrieving task list ...")

    task_filter = f"tag={task_tag}"
    tasks = gmp.get_tasks(filter_string=task_filter).xpath("task")
    print(f'Found {str(len(tasks))} task(s) with tag "{task_tag}".')

    for task in tasks:
        task_id = task.xpath("@id")[0]
        task_name = task.xpath("name/text()")[0]
        print(f'Processing task "{task_name}" ({task_id})...')

        reports = gmp.get_reports(
            filter_string=f"task_id={task_id} and status=Done "
            "sort-reverse=date"
        ).xpath("report")
        print(f"  Found {str(len(reports))} report(s).")
        if len(reports) < 2:
            print("  Delta-reporting requires at least 2 finished reports.")
            continue

        if reports[0].xpath(
            'report/user_tags/tag/name[text()="delta_alert_sent"]'
        ):
            print("  Delta report for latest finished report already sent")
            continue

        print(
            "  Latest finished report not send yet. Preparing delta "
            "report..."
        )

        delta_report = gmp.get_report(
            report_id=reports[0].xpath("@id")[0],
            delta_report_id=reports[1].xpath("@id")[0],
            filter_string="delta_states=n",
            format_id="c1645568-627a-11e3-a660-406186ea4fc5",
        )

        csv_in_b64 = delta_report.xpath("report/text()")[0]
        csv = base64.b64decode(csv_in_b64)

        print("  Composing Email...")
        alert_email = MIMEMultipart()
        alert_email["Subject"] = email_subject
        alert_email["To"] = ", ".join(to_addresses)
        alert_email["From"] = from_address
        alert_email["Date"] = formatdate(localtime=True)

        report_attachment = MIMEBase("application", "octet-stream")
        report_attachment.add_header(
            "Content-Disposition", "attachment", filename="delta.csv"
        )
        report_attachment.set_payload(csv)
        alert_email.attach(report_attachment)

        print("  Sending Email...")
        try:
            with smtplib.SMTP(mta_address, mta_port) as smtp:
                smtp.ehlo()
                smtp.starttls()
                smtp.ehlo()
                smtp.login(mta_user, mta_password)  # if required
                smtp.sendmail(
                    from_address, to_addresses, alert_email.as_string()
                )
                smtp.close()
                print("  Email has been sent!")

                gmp.create_tag(
                    name=report_tag_name,
                    resource_id=reports[0].xpath("@id")[0],
                    resource_type="report",
                    value=datetime.datetime.now(),
                )
        except Exception:  # pylint: disable=broad-except
            print("  Unable to send the email. Error: ", sys.exc_info()[0])
            # raise # in case an error should stop the script
            continue  # ignore the problem for the time being

    print(f"\nCheck will be repeated in {str(interval)} minutes...\n")
    sc.enter(
        interval * 60,
        1,
        execute_send_delta_emails,
        argument=(sc,),
        kwargs=kwargs,
    )


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable

    check_args(args)

    interval = 1  # in minutes
    task_tag = "send_delta_alert"
    report_tag_name = "delta_alert_sent"
    email_subject = "Delta Report"
    from_address = "admin@example.com"
    to_addresses = ["user1@example.com", "user2@example.com"]
    mta_address = "mail.example.com"
    mta_port = 25
    mta_user = "admin@example.com"
    mta_password = "mysecret"

    print("send_delta_alerts starting up with following settings:")
    print(f"User:          {args.username}")
    print(f"Interval:      {str(interval)} minutes")
    print(f"Task tag:      {task_tag}")
    print(f"Email subject: {email_subject}")
    print(f"From Address:  {from_address}")
    print(f"To Addresses:  {to_addresses}")
    print(f"MTA Address:   {mta_address}")
    print(f"MTA Port:      {str(mta_port)}")
    print(f"MTA User:      {mta_user}")
    print("MTA Password:  <will not be printed here>")
    print()

    print(f"Entering loop with interval {str(interval)} minutes ...")

    schedule = sched.scheduler(time.time, time.sleep)

    # Enter the scheduled execution with the given interval
    schedule.enter(
        0,
        1,
        execute_send_delta_emails,
        argument=(schedule,),
        kwargs={
            "gmp": gmp,
            "task_tag": task_tag,
            "interval": interval,
            "email_subject": email_subject,
            "to_addresses": to_addresses,
            "from_address": from_address,
            "mta_address": mta_address,
            "mta_password": mta_password,
            "mta_port": mta_port,
            "mta_user": mta_user,
            "report_tag_name": report_tag_name,
        },
    )
    schedule.run()


if __name__ == "__gmp__":
    main(gmp, args)



# -*- coding: utf-8 -*-
# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Command Line Interface Parser
"""

import argparse
import logging
from pathlib import Path
from typing import Optional

from gvm import get_version as get_gvm_version
from gvm.connections import (
    DEFAULT_TIMEOUT,
    SSHConnection,
    TLSConnection,
    UnixSocketConnection,
)

from gvmtools import get_version
from gvmtools.config import Config

logger = logging.getLogger(__name__)

__version__ = get_version()
__api_version__ = get_gvm_version()

DEFAULT_CONFIG_PATH = "~/.config/gvm-tools.conf"

PROTOCOL_OSP = "OSP"
PROTOCOL_GMP = "GMP"
DEFAULT_PROTOCOL = PROTOCOL_GMP


class CliParser:
    def __init__(
        self, description: str, logfilename, *, prog=None, ignore_config=False
    ):
        bootstrap_parser = argparse.ArgumentParser(
            prog=prog,
            description=description,
            formatter_class=argparse.RawTextHelpFormatter,
            # don't parse help initially. the args from parser wouldn't be shown
            add_help=False,
        )

        bootstrap_parser.add_argument(
            "-c",
            "--config",
            nargs="?",
            default=DEFAULT_CONFIG_PATH,
            help="Configuration file path (default: %(default)s)",
        )

        choices = ["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]

        bootstrap_parser.add_argument(
            "--log",
            nargs="?",
            dest="loglevel",
            const="INFO",
            type=lambda arg: {x.upper(): x for x in choices}[arg.upper()],
            choices=choices,
            help="Activate logging (default level: %(default)s)",
        )

        parser = argparse.ArgumentParser(prog=prog, parents=[bootstrap_parser])

        parser.add_argument(
            "--timeout",
            required=False,
            default=DEFAULT_TIMEOUT,
            type=int,
            help="Response timeout in seconds, or -1 to wait "
            "indefinitely (default: %(default)s)",
        )
        parser.add_argument(
            "--gmp-username",
            help="Username for GMP service (default: %(default)r)",
        )
        parser.add_argument(
            "--gmp-password",
            help="Password for GMP service (default: %(default)r)",
        )
        parser.add_argument(
            "-V",
            "--version",
            action="version",
            version=f"%(prog)s {__version__} (API version {__api_version__})",
            help="Show version information and exit",
        )

        subparsers = parser.add_subparsers(
            metavar="CONNECTION_TYPE",
            title="connections",
            description="valid connection types",
            help="Connection type to use",
        )
        subparsers.required = True
        subparsers.dest = "connection_type"

        self._subparsers = subparsers

        self._parser = parser
        self._bootstrap_parser = bootstrap_parser

        self._logfilename = logfilename
        self._ignore_config = ignore_config

        self._add_subparsers()

    def parse_args(self, args=None):
        args, unkown_args = self.parse_known_args(args)
        if unkown_args:
            self._parser.error(
                f'unrecognized arguments {" ".join(unkown_args)}'
            )
        return args

    def parse_known_args(self, args=None):
        args_before, _ = self._bootstrap_parser.parse_known_args(args)

        if args_before.loglevel is not None:
            level = logging.getLevelName(args_before.loglevel)
            logging.basicConfig(filename=self._logfilename, level=level)

        self._set_defaults(None if self._ignore_config else args_before.config)

        args, unknown_args = self._parser.parse_known_args(args)

        # If timeout value is -1, then the socket should have no timeout
        if args.timeout == -1:
            args.timeout = None

        logging.debug("Parsed arguments %r", args)

        return args, unknown_args

    def add_argument(self, *args, **kwargs):
        self._parser_socket.add_argument(*args, **kwargs)
        self._parser_ssh.add_argument(*args, **kwargs)
        self._parser_tls.add_argument(*args, **kwargs)

    def add_protocol_argument(self):
        self._parser.add_argument(
            "--protocol",
            required=False,
            default=DEFAULT_PROTOCOL,
            choices=[PROTOCOL_GMP, PROTOCOL_OSP],
            help="Service protocol to use (default: %(default)s)",
        )

    def _load_config(self, configfile):
        config = Config()

        if not configfile:
            return config

        configpath = Path(configfile)

        try:
            if not configpath.expanduser().resolve().exists():
                logger.debug("Ignoring non existing config file %s", configfile)
                return config
        except FileNotFoundError:
            # we are on python 3.5 and Path.resolve raised a FileNotFoundError
            logger.debug("Ignoring non existing config file %s", configfile)
            return config

        try:
            config.load(configpath)
            logger.debug("Loaded config %s", configfile)
        except Exception as e:  # pylint: disable=broad-except
            raise RuntimeError(
                f"Error while parsing config file {configfile}. Error was {e}"
            ) from None

        return config

    def _add_subparsers(self):
        parser_ssh = self._subparsers.add_parser(
            "ssh", help="Use SSH to connect to service"
        )

        parser_ssh.add_argument(
            "--hostname", help="Hostname or IP address (default: %(default)s)"
        )
        parser_ssh.add_argument(
            "--port",
            required=False,
            help="SSH port (default: %(default)s)",
            type=int,
        )
        parser_ssh.add_argument(
            "--ssh-username", help="SSH username (default: %(default)r)"
        )
        parser_ssh.add_argument(
            "--ssh-password", help="SSH password (default: %(default)r)"
        )
        parser_ssh.add_argument(
            "-A",
            "--auto-accept-host",
            action="store_true",
            help="When executed in e.g. CI, auto accept SSH host addition",
        )

        parser_tls = self._subparsers.add_parser(
            "tls", help="Use TLS secured connection to connect to service"
        )
        parser_tls.add_argument(
            "--hostname", help="Hostname or IP address (default: %(default)s)"
        )
        parser_tls.add_argument(
            "--port",
            required=False,
            help="GMP/OSP port (default: %(default)s)",
            type=int,
        )
        parser_tls.add_argument(
            "--certfile",
            required=False,
            help="Path to the certificate file for client authentication. "
            "(default: %(default)s)",
        )
        parser_tls.add_argument(
            "--keyfile",
            required=False,
            help="Path to key file for client authentication. "
            "(default: %(default)s)",
        )
        parser_tls.add_argument(
            "--cafile",
            required=False,
            help="Path to CA certificate for server authentication. "
            "(default: %(default)s)",
        )
        parser_tls.add_argument(
            "--no-credentials",
            required=False,
            default=False,
            action="store_true",
            help="Use only certificates for authentication",
        )

        parser_socket = self._subparsers.add_parser(
            "socket", help="Use UNIX Domain socket to connect to service"
        )

        socketpath_group = parser_socket.add_mutually_exclusive_group()
        socketpath_group.add_argument(
            "--sockpath",
            nargs="?",
            default=None,
            dest="socketpath",
            help="Deprecated, use --socketpath instead",
        )
        socketpath_group.add_argument(
            "--socketpath",
            nargs="?",
            help="Path to UNIX Domain socket (default: %(default)s)",
        )

        self._parser_ssh = parser_ssh
        self._parser_socket = parser_socket
        self._parser_tls = parser_tls

    def _set_defaults(self, configfilename=None):
        self._config = self._load_config(configfilename)

        self._parser.set_defaults(
            gmp_username=self._config.get("gmp", "username"),
            gmp_password=self._config.get("gmp", "password"),
            **self._config.defaults(),
        )

        self._parser_ssh.set_defaults(
            port=int(self._config.get("ssh", "port")),
            ssh_username=self._config.get("ssh", "username"),
            ssh_password=self._config.get("ssh", "password"),
            hostname=self._config.get("ssh", "hostname"),
        )
        self._parser_tls.set_defaults(
            port=int(self._config.get("tls", "port")),
            certfile=self._config.get("tls", "certfile"),
            keyfile=self._config.get("tls", "keyfile"),
            cafile=self._config.get("tls", "cafile"),
            hostname=self._config.get("tls", "hostname"),
        )
        self._parser_socket.set_defaults(
            socketpath=self._config.get("unixsocket", "socketpath")
        )


def create_parser(description, logfilename):
    return CliParser(description, logfilename)


def create_connection(
    connection_type,
    socketpath=None,
    timeout=None,
    hostname=None,
    port=None,
    certfile=None,
    keyfile=None,
    cafile=None,
    ssh_username=None,
    ssh_password=None,
    auto_accept_host: Optional[bool] = None,
    **kwargs,  # pylint: disable=unused-argument
):
    if "socket" in connection_type:
        return UnixSocketConnection(timeout=timeout, path=socketpath)

    if "tls" in connection_type:
        return TLSConnection(
            timeout=timeout,
            hostname=hostname,
            port=port,
            certfile=certfile,
            keyfile=keyfile,
            cafile=cafile,
        )

    return SSHConnection(
        timeout=timeout,
        hostname=hostname,
        port=port,
        username=ssh_username,
        password=ssh_password,
        auto_accept_host=auto_accept_host,
    )



# -*- coding: utf-8 -*-
# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import unittest
from argparse import Namespace
from pathlib import Path
from unittest.mock import patch

from lxml import etree

from . import GmpMockFactory, load_script

CWD = Path(__file__).absolute().parent


class CombineReportsTestCase(unittest.TestCase):
    def setUp(self):
        self.combine_reports = load_script(
            (CWD.parent.parent / "scripts"), "combine-reports"
        )

    @patch("gvm.protocols.latest.Gmp", new_callable=GmpMockFactory)
    def test_combine_reports(self, mock_gmp: GmpMockFactory):
        # bah!
        mock_gmp.mock_responses(
            "get_report",
            [
                '<get_reports_response status="200" status_text="OK">'
                '<report id="00000000-0000-0000-0000-000000000000">'
                "<name>2020-11-13T14:47:28Z</name>"
                "<creation_time>2020-11-13T14:47:28Z</creation_time>"
                "<modification_time>2020-11-13T14:47:28Z</modification_time>"
                '<task id="00000000-0000-0000-0001-000000000000">'
                "<name>Offline Scan from 2020-11-13T15:47:28+01:00 8</name>"
                "</task>"
                '<report id="00000000-0000-0000-0000-000000000000">'
                "<scan_run_status>Done</scan_run_status>"
                "<timestamp>2020-11-13T14:47:48Z</timestamp>"
                "<scan_start>2020-11-13T14:47:28Z</scan_start>"
                '<results start="1" max="100">'
                '<result id="00000001-0000-0000-0000-000000000000">'
                "</result>"
                '<result id="00000001-0000-0000-0000-000000000001">'
                "</result>"
                "</results>"
                "<scan_end>2020-11-13T14:47:28Z</scan_end>"
                "</report>"
                "</report>"
                "</get_reports_response>",
                '<get_reports_response status="200" status_text="OK">'
                '<report id="00000000-0000-0000-0000-000000000001">'
                "<name>2020-11-13T14:47:28Z</name>"
                "<creation_time>2020-11-13T14:47:28Z</creation_time>"
                "<modification_time>2020-11-13T14:47:28Z</modification_time>"
                '<task id="00000000-0000-0000-0002-000000000000">'
                "<name>Offline Scan from 2020-11-13T15:47:28+01:00 8</name>"
                "</task>"
                '<report id="00000000-0000-0000-0000-000000000001">'
                "<scan_run_status>Done</scan_run_status>"
                "<timestamp>2020-11-13T14:47:48Z</timestamp>"
                "<scan_start>2020-11-13T14:47:28Z</scan_start>"
                '<results start="1" max="100">'
                '<result id="00000001-0000-0000-0000-000000000002">'
                "</result>"
                '<result id="00000001-0000-0000-0000-000000000003">'
                "</result>"
                "</results>"
                "<scan_end>2020-11-13T14:47:28Z</scan_end>"
                "</report>"
                "</report>"
                "</get_reports_response>",
                '<get_reports_response status="200" status_text="OK">'
                '<report id="00000000-0000-0000-0000-000000000002">'
                "<name>2020-11-13T14:47:28Z</name>"
                "<creation_time>2020-11-13T14:47:28Z</creation_time>"
                "<modification_time>2020-11-13T14:47:28Z</modification_time>"
                '<task id="00000000-0000-0000-0003-000000000000">'
                "<name>Offline Scan from 2020-11-13T15:47:28+01:00 8</name>"
                "</task>"
                '<report id="00000000-0000-0000-0000-000000000002">'
                "<scan_run_status>Done</scan_run_status>"
                "<timestamp>2020-11-13T14:47:48Z</timestamp>"
                "<scan_start>2020-11-13T14:47:28Z</scan_start>"
                '<results start="1" max="100">'
                '<result id="00000001-0000-0000-0000-000000000004">'
                "</result>"
                "</results>"
                "<scan_end>2020-11-13T14:47:28Z</scan_end>"
                "</report>"
                "</report>"
                "</get_reports_response>",
            ],
        )
        args = Namespace(
            script=[
                "foo",
                "00000000-0000-0000-0000-000000000000",
                "00000000-0000-0000-0000-000000000001",
                "00000000-0000-0000-0000-000000000002",
            ]
        )

        combined_report = self.combine_reports.combine_reports(
            gmp=mock_gmp.gmp_protocol, args=args
        )

        results = (
            combined_report.find("report").find("results").findall("result")
        )
        i = 0
        for result in results:
            self.assertEqual(
                result.get("id"), f"00000001-0000-0000-0000-00000000000{str(i)}"
            )
            i += 1

        self.assertEqual(i, 5)

    @patch("gvm.protocols.latest.Gmp", new_callable=GmpMockFactory)
    def test_send_report(self, mock_gmp: GmpMockFactory):
        args = Namespace(
            script=[
                "foo",
                "00000000-0000-0000-0000-000000000000",
                "00000000-0000-0000-0000-000000000001",
                "00000000-0000-0000-0000-000000000002",
            ]
        )

        combined_report = etree.fromstring(
            '<report id="20574712-c404-4a04-9c83-03144ae02dca" '
            'format_id="d5da9f67-8551-4e51-807b-b6a873d70e34" '
            'extension="xml" content_type="text/xml">'
            '<report id="20574712-c404-4a04-9c83-03144ae02dca">'
            '<results start="1" max="-1">'
            '<result id="00000001-0000-0000-0000-000000000000"/>'
            '<result id="00000001-0000-0000-0000-000000000001"/>'
            '<result id="00000001-0000-0000-0000-000000000002"/>'
            '<result id="00000001-0000-0000-0000-000000000003"/>'
            '<result id="00000001-0000-0000-0000-000000000004"/>'
            "</results></report></report>"
        )

        report_id = "0e4d8fb2-47fa-494e-a242-d5327d3772f9"

        mock_gmp.mock_response(
            "import_report",
            '<create_report_response status="201" status_text="OK, '
            f'resource created" id="{report_id}"/>',
        )

        mock_gmp.mock_response(
            "create_container_task",
            '<create_task_response status="201" status_text="OK, '
            'resource created" id="6488ef71-e2d5-491f-95bd-ed9f915fa179"/>',
        )

        created_report_id = self.combine_reports.send_report(
            gmp=mock_gmp.gmp_protocol,
            args=args,
            combined_report=combined_report,
        )

        self.assertEqual(report_id, created_report_id)



# SPDX-FileCopyrightText: 2024 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

# Run with gvm-script --gmp-username admin-user --gmp-password password socket create-report_formats-from-csv.gmp.py report_formats.csv

import csv
import sys
from argparse import ArgumentParser, Namespace, RawTextHelpFormatter
from pathlib import Path

from gvm.protocols.gmp import Gmp
from gvmtools.helper import error_and_exit

HELP_TEXT = (
    "This script pulls report-format information "
    "from a csv file and creates a report-format for each row. \n"
    "csv file may contain Name of target, Login, password, and ssh-key \n"
    "Name,Type,Login,Password,ssh-key \n\n"
    "Please note: SNMP and ESX not supported yet "
)


def check_args(args):
    len_args = len(args.script) - 1
    if len_args != 2:
        message = """
        This script pulls report-format data from a csv file and creates a \
report-format for each row in the csv file.
        One parameter after the script name is required.

        1. <report_formats_csvfile>  -- csv file containing names and secrets required for scan report_formats

        Example:
            $ gvm-script --gmp-username name --gmp-password pass \
ssh --hostname <gsm> scripts/create_report_formats_from_csv.gmp.py \
<report_formats-csvfile>
        """
        print(message)
        sys.exit()


def parse_args(args: Namespace) -> Namespace:  # pylint: disable=unused-argument
    """Parsing args ..."""

    parser = ArgumentParser(
        prefix_chars="+",
        add_help=False,
        formatter_class=RawTextHelpFormatter,
        description=HELP_TEXT,
    )

    parser.add_argument(
        "+h",
        "++help",
        action="help",
        help="Show this help message and exit.",
    )

    parser.add_argument(
        "cred_file",
        type=str,
        help=("CSV File containing report_formats"),
    )
    script_args, _ = parser.parse_known_args(args)
    return script_args


def create_report_formats(
    gmp: Gmp,
    cred_file: Path,
):
    try:
        numberreport_formats = 0
        with open(cred_file, encoding="utf-8") as csvFile:
            content = csv.reader(csvFile, delimiter=",")  # read the data
            for row in content:  # loop through each row
                numberreport_formats = numberreport_formats + 1
                REPORT_FORMAT_XML_STRING = (
                    '<get_report_formats_response status="200" status_text="OK">'
                    '<report_format id="c4aa21e4-23e6-4064-ae49-c0d425738a98">'
                    "<name>CSV vulnscan.dk</name>"
                    "<term>name=CSV first=1 rows=10 sort=name</term>"
                    "<keywords>"
                    "<keyword>"
                    "<column>name</column>"
                    "<relation>=</relation>"
                    "<value>CSV</value>"
                    "</keyword>"
                    "<keyword>"
                    "<column>first</column>"
                    "<relation>=</relation>"
                    "<value>1</value>"
                    "</keyword>"
                    "<keyword>"
                    "<column>rows</column>"
                    "<relation>=</relation>"
                    "<value>10</value>"
                    "</keyword>"
                    "<keyword>"
                    "<column>sort</column>"
                    "<relation>=</relation>"
                    "<value>name</value>"
                    "</keyword>"
                    "</keywords>"
                    "<extension>csv</extension>"
                    "<content_type>text/csv</content_type>"
                    "<comment>vulnscan.dk CSV Report Format. Version 2023-03-31.</comment>"
                    "<summary>vulnscan.dk CSV Report Format</summary>"
                    "<description>Complete scan report in GMP XML format. Version 20200827.</description>"
                    "<predefined>1</predefined>"
                    "<configurable>0</configurable>"
                    "<trust>1<time>2024-03-30T18:24:54Z</time></trust>"
                    "<active>1</active>"
                    "<creation_time>2024-03-31T10:48:03Z</creation_time>"
                    "<modification_time>2024-03-31T10:48:03Z</modification_time>"
                    '<report_formats start="1" max="10000000"/>'
                    "<report_format_count>5<filtered>0</filtered><page>0</page></report_format_count>"
                    "</report_format>"
                    "</get_report_formats_response>"
                )

                gmp.import_report_format(REPORT_FORMAT_XML_STRING)

        csvFile.close()  # close the csv file

    except IOError as e:
        error_and_exit(f"Failed to read cred_file: {str(e)} (exit)")

    if len(row) == 0:
        error_and_exit("report_formats file is empty (exit)")

    return numberreport_formats


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable
    if args.script:
        args = args.script[1:]

    parsed_args = parse_args(args=args)

    numberreport_formats = create_report_formats(
        gmp,
        parsed_args.cred_file,
    )

    numberreport_formats = str(numberreport_formats)
    print("   \n [" + numberreport_formats + "] report_format(s) created!\n")


if __name__ == "__gmp__":
    main(gmp, args)



# -*- coding: utf-8 -*-
# SPDX-FileCopyrightText: 2019-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import logging
import unittest
from pathlib import Path

from gvm.connections import DEFAULT_GVM_PORT, DEFAULT_UNIX_SOCKET_PATH

from gvmtools.config import DEFAULT_SSH_PORT, Config
from gvmtools.config import __name__ as name

__here__ = Path(__file__).parent.resolve()


class ConfigTestCase(unittest.TestCase):
    def test_config_defaults(self):
        config = Config()

        self.assertEqual(config.get("gmp", "username"), "")
        self.assertEqual(config.get("gmp", "password"), "")

        self.assertEqual(config.get("ssh", "username"), "gmp")
        self.assertEqual(config.get("ssh", "password"), "gmp")
        self.assertEqual(config.get("ssh", "port"), DEFAULT_SSH_PORT)

        self.assertEqual(
            config.get("unixsocket", "socketpath"), DEFAULT_UNIX_SOCKET_PATH
        )

        self.assertEqual(config.get("tls", "port"), DEFAULT_GVM_PORT)

    def test_get_unknown_setting(self):
        config = Config()
        self.assertIsNone(config.get("foo", "bar"))

    def test_load(self):
        test_config_path = __here__ / "test.cfg"

        self.assertTrue(test_config_path.is_file())

        config = Config()
        config.load(test_config_path)

        self.assertEqual(config.get("gmp", "username"), "bar")
        self.assertEqual(config.get("gmp", "password"), "bar")

        self.assertEqual(config.get("ssh", "username"), "ipsum")
        self.assertEqual(config.get("ssh", "password"), "lorem")
        self.assertEqual(config.get("ssh", "port"), "123")

        self.assertEqual(
            config.get("unixsocket", "socketpath"), "/foo/bar.sock"
        )

        self.assertEqual(config.get("tls", "port"), "123")
        self.assertEqual(config.get("tls", "certfile"), "foo.cert")
        self.assertEqual(config.get("tls", "keyfile"), "foo.key")
        self.assertEqual(config.get("tls", "cafile"), "foo.ca")

        self.assertDictEqual(
            config.defaults(), dict(timeout="1000", foo="bar", username="ipsum")
        )

    def test_load_auth(self):
        root = logging.getLogger(name)
        root.disabled = True

        test_config_path = __here__ / "test_auth.cfg"

        self.assertTrue(test_config_path.is_file())

        config = Config()
        config.load(test_config_path)

        self.assertEqual(config.get("gmp", "username"), "foo")
        self.assertEqual(config.get("gmp", "password"), "bar")

        root.disabled = False

    def test_load_with_non_existing_configfile(self):
        test_config_path = __here__ / "foo.cfg"

        self.assertFalse(test_config_path.is_file())

        config = Config()

        with self.assertRaises(FileNotFoundError):
            config.load(test_config_path)



# SPDX-FileCopyrightText: 2018-2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import sys
from argparse import ArgumentParser, Namespace, RawTextHelpFormatter

from gvm.protocols.gmp import Gmp
from gvmtools.helper import create_xml_tree, error_and_exit, yes_or_no

HELP_TEXT = """
        This script pulls tasks data from an xml document and feeds it to \
            a desired GSM
        Usage examples:
            $ gvm-script --gmp-username name --gmp-password pass ssh --hostname
            ... send-task.gmp.py +h
            ... send-task.gmp.py ++x xml_file
    """


def numerical_option(statement, list_range):
    choice = int(input(statement))

    if choice in range(1, list_range + 1):
        return choice
    else:
        return numerical_option(
            f"Please enter valid number from 1 to {list_range}...",
            list_range,
        )


def interactive_options(gmp, task, keywords):
    options_dict = {}
    options_dict["config"] = gmp.get_scan_configs()
    options_dict["scanner"] = gmp.get_scanners()
    options_dict["target"] = gmp.get_targets()

    for option_key, option_value in options_dict.items():
        object_dict, object_list = {}, []
        object_id = task.find(option_key).get("id")
        object_xml = option_value

        for i in object_xml.findall(option_key):
            object_dict[i.find("name").text] = i.xpath("@id")[0]
            object_list.append(i.find("name").text)

        if object_id in object_dict.values():
            keywords[f"{option_key}_id"] = object_id
        elif object_id not in object_dict.values() and len(object_dict) != 0:
            response = yes_or_no(
                f"\nRequired Field: failed to detect {option_key}_id: "
                f"{task.xpath(f'{option_key}/@id')[0]}... "
                "\nWould you like to select from available options, or exit "
                "the script?"
            )

            if response is True:
                counter = 1
                print(f"{option_key.capitalize()} options:")
                for j in object_list:
                    print(f"    {counter} - {j}")
                    counter += 1
                answer = numerical_option(
                    "\nPlease enter the number of your choice.",
                    len(object_list),
                )
                keywords[f"{option_key}_id"] = object_dict[
                    object_list[answer - 1]
                ]
            else:
                print("\nTerminating...")
                sys.exit()
        else:
            error_and_exit(
                f"Failed to detect {option_key}_id\nThis field is required "
                "therefore the script is unable to continue.\n"
            )


def parse_send_xml_tree(gmp, xml_tree):
    task_xml_elements = xml_tree.xpath("task")
    print(task_xml_elements)
    if not task_xml_elements:
        error_and_exit("No tasks found.")
    tasks = []
    for task in task_xml_elements:
        keywords = {"name": task.find("name").text}

        if task.find("comment").text is not None:
            keywords["comment"] = task.find("comment").text

        interactive_options(gmp, task, keywords)

        if task.find("schedule_periods") is not None:
            keywords["schedule_periods"] = int(
                task.find("schedule_periods").text
            )

        if task.find("observers").text:
            keywords["observers"] = task.find("observers").text

        if task.xpath("schedule/@id")[0]:
            keywords["schedule_id"] = task.xpath("schedule/@id")[0]

        if task.xpath("preferences/preference"):
            preferences, scanner_name_list, value_list = {}, [], []

            for preference in task.xpath("preferences/preference"):
                scanner_name_list.append(preference.find("scanner_name").text)
                if preference.find("value").text is not None:
                    value_list.append(preference.find("value").text)
                else:
                    value_list.append("")
            preferences["scanner_name"] = scanner_name_list
            preferences["value"] = value_list
            keywords["preferences"] = preferences

        new_task = gmp.create_task(**keywords)

        tasks.append(new_task.xpath("//@id")[0])
    return tasks


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable, unused-argument

    parser = ArgumentParser(
        prefix_chars="+",
        add_help=False,
        formatter_class=RawTextHelpFormatter,
        description=HELP_TEXT,
    )

    parser.add_argument(
        "+h",
        "++help",
        action="help",
        help="Show this help message and exit.",
    )

    parser.add_argument(
        "+x",
        "++xml-file",
        dest="xml",
        type=str,
        required=True,
        help="xml file containing tasks",
    )

    script_args, _ = parser.parse_known_args()

    # check_args(args)

    print("\nSending task(s)...")

    xml_tree = create_xml_tree(script_args.xml)
    tasks = parse_send_xml_tree(gmp, xml_tree)
    for task in tasks:
        print(task)
    print("\nTask(s) sent!\n")


if __name__ == "__gmp__":
    main(gmp, args)  # pylint: disable=undefined-variable



# SPDX-FileCopyrightText: 2024 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later

from argparse import Namespace

from gvm.protocols.gmp import Gmp
from gvmtools.helper import Table


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=unused-argument

    response_xml = gmp.get_scanners(filter_string="rows=-1")
    scanners_xml = response_xml.xpath("scanner")

    heading = ["#", "Name", "Id", "host"]

    rows = []
    numberRows = 0

    print("Listing scanners.\n")

    for scanner in scanners_xml:
        # Count number of reports
        numberRows = numberRows + 1
        # Cast/convert to text to show in list
        rowNumber = str(numberRows)

        name = "".join(scanner.xpath("name/text()"))
        scanner_id = scanner.get("id")
        host = "".join(scanner.xpath("host/text()"))

        rows.append([rowNumber, name, scanner_id, host])

    print(Table(heading=heading, rows=rows))


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2019-2021 Greenbone AG
# SPDX-FileCopyrightText: 2024 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later

import sys
from argparse import Namespace
from base64 import b64decode
from pathlib import Path

from gvm.protocols.gmp import Gmp


def check_args(args):
    len_args = len(args.script) - 1
    if len_args < 1:
        message = """
        This script requests the given report and exports it as a pdf
        file locally. It requires one parameters after the script name.

        1. <report_id>     -- ID of the report

        Optional a file name to save the pdf in.

        Example:
            $ gvm-script --gmp-username name --gmp-password pass \
ssh --hostname <gsm> scripts/export-pdf-report.gmp.py <report_id> <pdf_file>
        """
        print(message)
        sys.exit()


def main(gmp: Gmp, args: Namespace) -> None:
    # check if report id and PDF filename are provided to the script
    # argv[0] contains the script name
    check_args(args)

    report_id = args.argv[1]
    if len(args.argv) == 3:
        pdf_filename = args.argv[2] + ".pdf"
    else:
        pdf_filename = args.argv[1] + ".pdf"

    pdf_report_format_id = "c402cc3e-b531-11e1-9163-406186ea4fc5"

    response = gmp.get_report(
        report_id=report_id,
        report_format_id=pdf_report_format_id,
        ignore_pagination=True,
        details=True,
    )

    report_element = response.find("report")
    # get the full content of the report element
    content = report_element.find("report_format").tail

    if not content:
        print(
            "Requested report is empty. Either the report does not contain any "
            "results or the necessary tools for creating the report are "
            "not installed.",
            file=sys.stderr,
        )
        sys.exit(1)

    # convert content to 8-bit ASCII bytes
    binary_base64_encoded_pdf = content.encode("ascii")

    # decode base64
    binary_pdf = b64decode(binary_base64_encoded_pdf)

    # write to file and support ~ in filename path
    pdf_path = Path(pdf_filename).expanduser()

    pdf_path.write_bytes(binary_pdf)

    print("Done. PDF created: " + str(pdf_path))


if __name__ == "__gmp__":
    main(gmp, args)



# -*- coding: utf-8 -*-
# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import code
import logging
import os
import sys
from argparse import Namespace

from gvm import get_version as get_gvm_version
from gvm.protocols.gmp import Gmp
from gvm.protocols.latest import Osp
from gvm.transforms import EtreeCheckCommandTransform
from gvm.xml import pretty_print

from gvmtools import get_version
from gvmtools.helper import authenticate, do_not_run_as_root, run_script
from gvmtools.parser import (
    PROTOCOL_GMP,
    PROTOCOL_OSP,
    create_connection,
    create_parser,
)

__version__ = get_version()
__api_version__ = get_gvm_version()

logger = logging.getLogger(__name__)

HELP_TEXT = """
    Command line tool to access services via GMP (Greenbone Management
    Protocol) and OSP (Open Scanner Protocol)

    gvm-pyshell provides an interactive shell for GMP and OSP services.

    Example:
        >>> tasks = gmp.get_tasks()
        >>> task_names = tasks.xpath('task/name/text()')
        >>> print(task_names)
        ['Scan Task']
        >>> pretty_print('<xml/>')
        <xml/>

    The interactive shell can be exited with:
        Ctrl + D on Linux  or
        Ctrl + Z on Windows

    The protocol specifications for GMP and OSP are available at:
      https://docs.greenbone.net/index.html#api_documentation"""


class Help(object):
    """Help class to overwrite the help function from python itself."""

    def __call__(self):
        return print(HELP_TEXT)

    def __repr__(self):
        # do pwd command
        return HELP_TEXT


def main():
    do_not_run_as_root()

    parser = create_parser(description=HELP_TEXT, logfilename="gvm-pyshell.log")

    parser.add_protocol_argument()

    parser.add_argument(
        "-i",
        "--interactive",
        action="store_true",
        default=False,
        help="Start an interactive Python shell",
    )

    parser.add_argument(
        "scriptname",
        nargs="?",
        metavar="SCRIPT",
        help="Path to script to be preloaded (example: myscript.gmp.py)",
    )
    parser.add_argument(
        "scriptargs",
        nargs="*",
        metavar="ARG",
        help="Arguments for preloaded script",
    )

    args = parser.parse_args()

    connection = create_connection(**vars(args))

    transform = EtreeCheckCommandTransform()

    global_vars = {
        "help": Help(),
        "pretty_print": pretty_print,
        "__version__": __version__,
        "__api_version__": __api_version__,
    }

    username = None
    password = None

    if args.protocol == PROTOCOL_OSP:
        protocol_class = Osp
        name = "osp"
    else:
        protocol_class = Gmp
        name = "gmp"

    with protocol_class(connection, transform=transform) as protocol:
        global_vars[name] = protocol
        global_vars["__name__"] = f"__{name}__"

        if args.protocol == PROTOCOL_GMP:
            if args.gmp_username:
                (username, password) = authenticate(
                    protocol,
                    username=args.gmp_username,
                    password=args.gmp_password,
                )

        shell_args = Namespace(username=username, password=password)

        global_vars["args"] = shell_args

        with_script = args.scriptname and len(args.scriptname) > 0

        if with_script:
            argv = [os.path.abspath(args.scriptname), *args.scriptargs]
            shell_args.argv = argv
            # for backwards compatibility we add script here
            shell_args.script = argv

        no_script_no_interactive = not args.interactive and not with_script
        script_and_interactive = args.interactive and with_script
        only_interactive = not with_script and args.interactive
        only_script = not args.interactive and with_script

        if only_interactive or no_script_no_interactive:
            enter_interactive_mode(global_vars)

        if script_and_interactive or only_script:
            if only_script:
                print(
                    "Using gvm-pyshell for running scripts only is deprecated. "
                    "Please use gvm-script instead",
                    file=sys.stderr,
                )

            script_name = args.scriptname
            run_script(script_name, global_vars)

            if not only_script:
                enter_interactive_mode(global_vars)


def enter_interactive_mode(global_vars):
    code.interact(
        banner=f"GVM Interactive Console {__version__} API {__api_version__}."
        'Type "help" to get information about functionality.',
        local=dict(global_vars),
    )


if __name__ == "__main__":
    main()



# SPDX-FileCopyrightText: 2017-2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import sys
from argparse import Namespace
from datetime import date, timedelta

from gvm.protocols.gmp import Gmp
from lxml.etree import Element
from terminaltables import AsciiTable


def check_args(args: Namespace) -> None:
    len_args = len(args.script) - 1
    if len_args < 2:
        message = """
        This script will display all vulnerabilities from the hosts of the
        reports in a given month!

        1. <month>  -- month of the monthly report
        2. <year>   -- year of the monthly report

        The third is 'with-tables' parameter to activate a verbose output of
        hosts. Explicitly made for GOS 3.1.

        Example:
            $ gvm-script --gmp-username name --gmp-password pass \
    ssh --hostname <gsm> scripts/monthly-report.gmp.py 05 2017 with-tables
        """
        print(message)
        sys.exit()


def get_reports_xml(gmp: Gmp, from_date: date, to_date: date) -> Element:
    """Getting the Reports in the defined time period"""

    report_filter = (
        f"rows=-1 created>{from_date.isoformat()} and "
        f"created<{to_date.isoformat()}"
    )

    return gmp.get_reports(filter_string=report_filter)


def print_result_sums(
    reports_xml: Element, from_date: date, to_date: date
) -> None:
    report_count = len(reports_xml.xpath("report"))
    print(f"Found {report_count} reports")

    sum_high = reports_xml.xpath(
        "sum(report/report/result_count/hole/full/text())"
    )
    sum_medium = reports_xml.xpath(
        "sum(report/report/result_count/warning/full/text())"
    )
    sum_low = reports_xml.xpath(
        "sum(report/report/result_count/info/full/text())"
    )

    print(
        f"Summary of results from {from_date.isoformat()} "
        f"to {to_date.isoformat()}"
    )
    print(f"High: {int(sum_high)}")
    print(f"Medium: {int(sum_medium)}")
    print(f"Low: {int(sum_low)}")


def print_result_tables(gmp: Gmp, reports_xml: Element) -> None:
    report_list = reports_xml.xpath("report")

    for report in report_list:
        report_id = report.xpath("report/@id")[0]
        name = report.xpath("name/text()")[0]

        res = gmp.get_report(report_id)

        print(f"\nReport: {report_id}")

        table_data = [["Hostname", "IP", "Bericht", "high", "medium", "low"]]

        for host in res.xpath("report/report/host"):
            hostname = host.xpath(
                'detail/name[text()="hostname"]/../value/text()'
            )
            if len(hostname) > 0:
                hostname = str(hostname[0])
            else:
                hostname = ""

            ip = host.xpath("ip/text()")[0]
            high = host.xpath("result_count/hole/page/text()")[0]
            medium = host.xpath("result_count/warning/page/text()")[0]
            low = host.xpath("result_count/info/page/text()")[0]

            table_data.append([hostname, ip, name, high, medium, low])

        table = AsciiTable(table_data)
        print(table.table + "\n")


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable

    check_args(args)

    month = int(args.script[1])
    year = int(args.script[2])
    from_date = date(year, month, 1)
    to_date = from_date + timedelta(days=31)
    # To have the first day in month
    to_date = to_date.replace(day=1)

    reports_xml = get_reports_xml(gmp, from_date, to_date)

    print_result_sums(reports_xml, from_date, to_date)
    if "with-tables" in args.script:
        print_result_tables(gmp, reports_xml)


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2024 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later

from argparse import Namespace

from gvm.protocols.gmp import Gmp
from gvmtools.helper import Table


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=unused-argument

    response_xml = gmp.get_tasks(details=True, filter_string="rows=-1")
    tasks_xml = response_xml.xpath("task")

    heading = ["#", "Name", "Id", "Target", "Scanner", "Scan Order", "Severity"]

    rows = []
    numberRows = 0

    print("Listing tasks.\n")

    for task in tasks_xml:
        # Count number of reports
        numberRows = numberRows + 1
        # Cast/convert to text to show in list
        rowNumber = str(numberRows)

        name = "".join(task.xpath("name/text()"))
        task_id = task.get("id")
        targetname = "".join(task.xpath("target/name/text()"))
        scanner = "".join(task.xpath("scanner/name/text()"))
        severity = "".join(task.xpath("last_report/report/severity/text()"))
        order = "".join(task.xpath("hosts_ordering/text()"))
        rows.append(
            [rowNumber, name, task_id, targetname, scanner, order, severity]
        )

    print(Table(heading=heading, rows=rows))


if __name__ == "__gmp__":
    main(gmp, args)



# -*- coding: utf-8 -*-
# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import unittest
from datetime import date
from pathlib import Path
from unittest.mock import patch

from lxml import etree

from . import GmpMockFactory, load_script

CWD = Path(__file__).absolute().parent


class CreateConsolidatedReportsTestCase(unittest.TestCase):
    def setUp(self):
        self.create_consolidated_report = load_script(
            (CWD.parent.parent / "scripts"), "create-consolidated-report"
        )

    def test_parse_period(self):
        with self.assertRaises(SystemExit), self.assertRaises(ValueError):
            self.create_consolidated_report.parse_period(
                ["a/b/c", "2020/02/03"]
            )

        with self.assertRaises(SystemExit), self.assertRaises(ValueError):
            self.create_consolidated_report.parse_period(
                ["1/2/3/4", "2020/02/03"]
            )

        with self.assertRaises(SystemExit), self.assertRaises(ValueError):
            self.create_consolidated_report.parse_period(
                ["2020/02/03", "a/b/c"]
            )

        with self.assertRaises(SystemExit), self.assertRaises(ValueError):
            self.create_consolidated_report.parse_period(
                ["2020/02/03", "1/2/3/4"]
            )

        with self.assertRaises(SystemExit), self.assertRaises(ValueError):
            self.create_consolidated_report.parse_period(
                ["2020/20/03", "2001/2/3"]
            )

        with self.assertRaises(SystemExit), self.assertRaises(ValueError):
            self.create_consolidated_report.parse_period(
                ["2020/12/03", "2001/2/300"]
            )

        with self.assertRaises(SystemExit), self.assertRaises(ValueError):
            self.create_consolidated_report.parse_period(
                ["2020/12/03", "2001/22/30"]
            )

        with self.assertRaises(SystemExit), self.assertRaises(ValueError):
            self.create_consolidated_report.parse_period(
                ["2020/12/43", "2001/2/3"]
            )

        date1, date2 = self.create_consolidated_report.parse_period(
            ["1980/1/11", "2001/2/3"]
        )
        self.assertEqual(date1, date(1980, 1, 11))
        self.assertEqual(date2, date(2001, 2, 3))

    def test_parse_tags(self):
        tags = ["abc", "dba8624e-a56c-4901-a3f2-591f062e4c20"]

        filter_tags = self.create_consolidated_report.parse_tags(tags)

        self.assertEqual(filter_tags[0], 'tag="abc"')
        self.assertEqual(
            filter_tags[1], 'tag_id="dba8624e-a56c-4901-a3f2-591f062e4c20"'
        )

    def test_generate_task_filter(self):
        asserted_task_filter = (
            'rows=-1 last>2020-01-01 and created<2020-02-01 and tag="blah"'
        )
        period_start = date(2020, 1, 1)
        period_end = date(2020, 2, 1)
        tags = ['tag="blah"']

        task_filter = self.create_consolidated_report.generate_task_filter(
            period_start, period_end, tags
        )

        self.assertEqual(task_filter, asserted_task_filter)

        asserted_task_filter = (
            "rows=-1 last>2020-01-01 and created<2020-02-01 and "
            'tag="blah" or last>2020-01-01 and created<2020-02-01'
            ' and tag="blah2"'
        )
        tags = ['tag="blah"', 'tag="blah2"']

        task_filter = self.create_consolidated_report.generate_task_filter(
            period_start, period_end, tags
        )

        self.assertEqual(task_filter, asserted_task_filter)

    @patch("gvm.protocols.latest.Gmp", new_callable=GmpMockFactory)
    def test_combine_reports_with_term(self, mock_gmp: GmpMockFactory):
        reports = [
            "00000000-0000-0000-0000-000000000000",
            "00000000-0000-0000-0000-000000000001",
            "00000000-0000-0000-0000-000000000002",
        ]

        mock_gmp.mock_responses(
            "get_report",
            [
                '<get_reports_response status="200" status_text="OK">'
                '<report id="00000000-0000-0000-0000-000000000000">'
                "<name>2020-11-13T14:47:28Z</name>"
                "<creation_time>2020-11-13T14:47:28Z</creation_time>"
                "<modification_time>2020-11-13T14:47:28Z</modification_time>"
                '<task id="00000000-0000-0000-0001-000000000000">'
                "<name>Offline Scan from 2020-11-13T15:47:28+01:00 8</name>"
                "</task>"
                '<report id="00000000-0000-0000-0000-000000000000">'
                "<scan_run_status>Done</scan_run_status>"
                "<timestamp>2020-11-13T14:47:48Z</timestamp>"
                "<scan_start>2020-11-13T14:47:28Z</scan_start>"
                '<ports max="-1" start="1">'
                "<port>0<host>127.0.0.0</host></port>"
                "<port>1<host>127.0.0.1</host></port></ports>"
                '<results start="1" max="100">'
                '<result id="00000001-0000-0000-0000-000000000000">'
                "</result>"
                '<result id="00000001-0000-0000-0000-000000000001">'
                "</result></results>"
                "<scan_end>2020-11-13T14:47:28Z</scan_end>"
                "</report></report></get_reports_response>",
                '<get_reports_response status="200" status_text="OK">'
                '<report id="00000000-0000-0000-0000-000000000001">'
                "<name>2020-11-13T14:47:28Z</name>"
                "<creation_time>2020-11-13T14:47:28Z</creation_time>"
                "<modification_time>2020-11-13T14:47:28Z</modification_time>"
                '<task id="00000000-0000-0000-0002-000000000000">'
                "<name>Offline Scan from 2020-11-13T15:47:28+01:00 8</name>"
                "</task>"
                '<report id="00000000-0000-0000-0000-000000000001">'
                "<scan_run_status>Done</scan_run_status>"
                "<timestamp>2020-11-13T14:47:48Z</timestamp>"
                "<scan_start>2020-11-13T14:47:28Z</scan_start>"
                '<ports max="-1" start="1">'
                "<port>2<host>127.0.0.2</host></port>"
                "<port>3<host>127.0.0.3</host></port></ports>"
                '<results start="1" max="100">'
                '<result id="00000001-0000-0000-0000-000000000002"></result>'
                '<result id="00000001-0000-0000-0000-000000000003">'
                "</result></results>"
                "<scan_end>2020-11-13T14:47:28Z</scan_end>"
                "<host><ip>127.0.0.0</ip></host></report>"
                "</report></get_reports_response>",
                '<get_reports_response status="200" status_text="OK">'
                '<report id="00000000-0000-0000-0000-000000000002">'
                "<name>2020-11-13T14:47:28Z</name>"
                "<creation_time>2020-11-13T14:47:28Z</creation_time>"
                "<modification_time>2020-11-13T14:47:28Z</modification_time>"
                '<task id="00000000-0000-0000-0003-000000000000">'
                "<name>Offline Scan from 2020-11-13T15:47:28+01:00 8</name>"
                "</task>"
                '<report id="00000000-0000-0000-0000-000000000002">'
                "<scan_run_status>Done</scan_run_status>"
                "<timestamp>2020-11-13T14:47:48Z</timestamp>"
                "<scan_start>2020-11-13T14:47:28Z</scan_start>"
                '<results start="1" max="100">'
                '<result id="00000001-0000-0000-0000-000000000004">'
                "</result></results>"
                "<scan_end>2020-11-13T14:47:28Z</scan_end>"
                "<host><ip>127.0.0.1</ip></host></report>"
                "</report></get_reports_response>",
            ],
        )

        combined_report = self.create_consolidated_report.combine_reports(
            mock_gmp.gmp_protocol, reports, filter_term="foo", filter_id=None
        )

        ports = combined_report.find("report").find("ports").findall("port")
        i = 0
        for port in ports:
            self.assertEqual(port.text, f"{str(i)}")
            i += 1

        self.assertEqual(i, 4)

        results = (
            combined_report.find("report").find("results").findall("result")
        )
        i = 0
        for result in results:
            self.assertEqual(
                result.get("id"), f"00000001-0000-0000-0000-00000000000{str(i)}"
            )
            i += 1

        self.assertEqual(i, 5)

        hosts = combined_report.find("report").findall("host")

        i = 0
        for host in hosts:
            self.assertEqual(host.find("ip").text, f"127.0.0.{str(i)}")
            i += 1

        self.assertEqual(i, 2)

    @patch("gvm.protocols.latest.Gmp", new_callable=GmpMockFactory)
    def test_combine_reports_with_id(self, mock_gmp: GmpMockFactory):
        reports = [
            "00000000-0000-0000-0000-000000000000",
            "00000000-0000-0000-0000-000000000001",
            "00000000-0000-0000-0000-000000000002",
        ]

        mock_gmp.mock_responses(
            "get_report",
            [
                '<get_reports_response status="200" status_text="OK">'
                '<report id="00000000-0000-0000-0000-000000000000">'
                "<name>2020-11-13T14:47:28Z</name>"
                "<creation_time>2020-11-13T14:47:28Z</creation_time>"
                "<modification_time>2020-11-13T14:47:28Z</modification_time>"
                '<task id="00000000-0000-0000-0001-000000000000">'
                "<name>Offline Scan from 2020-11-13T15:47:28+01:00 8</name>"
                "</task>"
                '<report id="00000000-0000-0000-0000-000000000000">'
                "<scan_run_status>Done</scan_run_status>"
                "<timestamp>2020-11-13T14:47:48Z</timestamp>"
                "<scan_start>2020-11-13T14:47:28Z</scan_start>"
                '<ports max="-1" start="1">'
                "<port>0<host>127.0.0.0</host></port>"
                "<port>1<host>127.0.0.1</host></port></ports>"
                '<results start="1" max="100">'
                '<result id="00000001-0000-0000-0000-000000000000">'
                "</result>"
                '<result id="00000001-0000-0000-0000-000000000001">'
                "</result></results>"
                "<scan_end>2020-11-13T14:47:28Z</scan_end>"
                "</report></report></get_reports_response>",
                '<get_reports_response status="200" status_text="OK">'
                '<report id="00000000-0000-0000-0000-000000000001">'
                "<name>2020-11-13T14:47:28Z</name>"
                "<creation_time>2020-11-13T14:47:28Z</creation_time>"
                "<modification_time>2020-11-13T14:47:28Z</modification_time>"
                '<task id="00000000-0000-0000-0002-000000000000">'
                "<name>Offline Scan from 2020-11-13T15:47:28+01:00 8</name>"
                "</task>"
                '<report id="00000000-0000-0000-0000-000000000001">'
                "<scan_run_status>Done</scan_run_status>"
                "<timestamp>2020-11-13T14:47:48Z</timestamp>"
                "<scan_start>2020-11-13T14:47:28Z</scan_start>"
                '<ports max="-1" start="1">'
                "<port>2<host>127.0.0.2</host></port>"
                "<port>3<host>127.0.0.3</host></port></ports>"
                '<results start="1" max="100">'
                '<result id="00000001-0000-0000-0000-000000000002"></result>'
                '<result id="00000001-0000-0000-0000-000000000003">'
                "</result></results>"
                "<scan_end>2020-11-13T14:47:28Z</scan_end>"
                "<host><ip>127.0.0.0</ip></host></report>"
                "</report></get_reports_response>",
                '<get_reports_response status="200" status_text="OK">'
                '<report id="00000000-0000-0000-0000-000000000002">'
                "<name>2020-11-13T14:47:28Z</name>"
                "<creation_time>2020-11-13T14:47:28Z</creation_time>"
                "<modification_time>2020-11-13T14:47:28Z</modification_time>"
                '<task id="00000000-0000-0000-0003-000000000000">'
                "<name>Offline Scan from 2020-11-13T15:47:28+01:00 8</name>"
                "</task>"
                '<report id="00000000-0000-0000-0000-000000000002">'
                "<scan_run_status>Done</scan_run_status>"
                "<timestamp>2020-11-13T14:47:48Z</timestamp>"
                "<scan_start>2020-11-13T14:47:28Z</scan_start>"
                '<results start="1" max="100">'
                '<result id="00000001-0000-0000-0000-000000000004">'
                "</result></results>"
                "<scan_end>2020-11-13T14:47:28Z</scan_end>"
                "<host><ip>127.0.0.1</ip></host></report>"
                "</report></get_reports_response>",
            ],
        )

        combined_report = self.create_consolidated_report.combine_reports(
            mock_gmp.gmp_protocol, reports, filter_term="", filter_id="123"
        )

        ports = combined_report.find("report").find("ports").findall("port")
        i = 0
        for port in ports:
            self.assertEqual(port.text, f"{str(i)}")
            i += 1

        self.assertEqual(i, 4)

        results = (
            combined_report.find("report").find("results").findall("result")
        )
        i = 0
        for result in results:
            self.assertEqual(
                result.get("id"), f"00000001-0000-0000-0000-00000000000{str(i)}"
            )
            i += 1

        self.assertEqual(i, 5)

        hosts = combined_report.find("report").findall("host")

        i = 0
        for host in hosts:
            self.assertEqual(host.find("ip").text, f"127.0.0.{str(i)}")
            i += 1

        self.assertEqual(i, 2)

    @patch("gvm.protocols.latest.Gmp", new_callable=GmpMockFactory)
    def test_send_report(self, mock_gmp: GmpMockFactory):
        combined_report = etree.fromstring(
            '<report id="20574712-c404-4a04-9c83-03144ae02dca" '
            'format_id="d5da9f67-8551-4e51-807b-b6a873d70e34" '
            'extension="xml" content_type="text/xml">'
            '<report id="20574712-c404-4a04-9c83-03144ae02dca">'
            '<results start="1" max="-1">'
            '<result id="00000001-0000-0000-0000-000000000000"/>'
            '<result id="00000001-0000-0000-0000-000000000001"/>'
            '<result id="00000001-0000-0000-0000-000000000002"/>'
            '<result id="00000001-0000-0000-0000-000000000003"/>'
            '<result id="00000001-0000-0000-0000-000000000004"/>'
            "</results></report></report>"
        )

        report_id = "0e4d8fb2-47fa-494e-a242-d5327d3772f9"

        mock_gmp.mock_response(
            "import_report",
            '<create_report_response status="201" status_text="OK, '
            f'resource created" id="{report_id}"/>',
        )

        mock_gmp.mock_response(
            "create_container_task",
            '<create_task_response status="201" status_text="OK, '
            'resource created" id="6488ef71-e2d5-491f-95bd-ed9f915fa179"/>',
        )

        period_start = date(2020, 1, 1)
        period_end = date(2020, 2, 1)

        created_report_id = self.create_consolidated_report.send_report(
            gmp=mock_gmp.gmp_protocol,
            combined_report=combined_report,
            period_start=period_start,
            period_end=period_end,
            container_id=None,
            new_container_name=None,
        )

        self.assertEqual(report_id, created_report_id)

    @patch("gvm.protocols.latest.Gmp", new_callable=GmpMockFactory)
    def test_send_report_with_container_id(self, mock_gmp: GmpMockFactory):
        combined_report = etree.fromstring(
            '<report id="20574712-c404-4a04-9c83-03144ae02dca" '
            'format_id="d5da9f67-8551-4e51-807b-b6a873d70e34" '
            'extension="xml" content_type="text/xml">'
            '<report id="20574712-c404-4a04-9c83-03144ae02dca">'
            '<results start="1" max="-1">'
            '<result id="00000001-0000-0000-0000-000000000000"/>'
            '<result id="00000001-0000-0000-0000-000000000001"/>'
            '<result id="00000001-0000-0000-0000-000000000002"/>'
            '<result id="00000001-0000-0000-0000-000000000003"/>'
            '<result id="00000001-0000-0000-0000-000000000004"/>'
            "</results></report></report>"
        )

        task_id = "c347836e-3c51-4045-872d-3cb12637f4cc"
        report_id = "0e4d8fb2-47fa-494e-a242-d5327d3772f9"

        mock_gmp.mock_response(
            "import_report",
            '<create_report_response status="201" status_text="OK, '
            f'resource created" id="{report_id}"/>',
        )

        # Returns a container task without a target
        mock_gmp.mock_response(
            "get_task",
            '<get_tasks_response status="200" status_text="OK">'
            '<task id="c347836e-3c51-4045-872d-3cb12637f4cc">'
            "<name>test</name>"
            '<target id=""/>'
            "</task>"
            "</get_tasks_response>",
        )

        period_start = date(2020, 1, 1)
        period_end = date(2020, 2, 1)

        created_report_id = self.create_consolidated_report.send_report(
            gmp=mock_gmp.gmp_protocol,
            combined_report=combined_report,
            period_start=period_start,
            period_end=period_end,
            container_id=task_id,
            new_container_name="test",
        )

        self.assertEqual(report_id, created_report_id)

    @patch("gvm.protocols.latest.Gmp", new_callable=GmpMockFactory)
    def test_send_report_with_container_id_failure(
        self, mock_gmp: GmpMockFactory
    ):
        combined_report = etree.fromstring(
            '<report id="20574712-c404-4a04-9c83-03144ae02dca" '
            'format_id="d5da9f67-8551-4e51-807b-b6a873d70e34" '
            'extension="xml" content_type="text/xml">'
            '<report id="20574712-c404-4a04-9c83-03144ae02dca">'
            '<results start="1" max="-1">'
            '<result id="00000001-0000-0000-0000-000000000000"/>'
            '<result id="00000001-0000-0000-0000-000000000001"/>'
            '<result id="00000001-0000-0000-0000-000000000002"/>'
            '<result id="00000001-0000-0000-0000-000000000003"/>'
            '<result id="00000001-0000-0000-0000-000000000004"/>'
            "</results></report></report>"
        )

        task_id = "c347836e-3c51-4045-872d-3cb12637f4cc"
        report_id = "0e4d8fb2-47fa-494e-a242-d5327d3772f9"

        mock_gmp.mock_response(
            "import_report",
            '<create_report_response status="201" status_text="OK, '
            f'resource created" id="{report_id}"/>',
        )

        # Returns a non-container task with a target defined
        mock_gmp.mock_response(
            "get_task",
            '<get_tasks_response status="200" status_text="OK">'
            '<task id="c347836e-3c51-4045-872d-3cb12637f4cc">'
            "<name>test</name>"
            '<target id="35995ae0-2430-4f0d-97da-fcb93350abb4"/>'
            "</task>"
            "</get_tasks_response>",
        )

        period_start = date(2020, 1, 1)
        period_end = date(2020, 2, 1)

        with self.assertRaises(SystemExit):
            self.create_consolidated_report.send_report(
                gmp=mock_gmp.gmp_protocol,
                combined_report=combined_report,
                period_start=period_start,
                period_end=period_end,
                container_id=task_id,
                new_container_name="test",
            )



# SPDX-FileCopyrightText: 2024 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

# Run with gvm-script --gmp-username admin-user --gmp-password password socket create-targets-from-csv.gmp.py hostname-server targets.csv

import csv
import sys
import time
from argparse import ArgumentParser, Namespace, RawTextHelpFormatter
from pathlib import Path

from gvm.errors import GvmResponseError
from gvm.protocols.gmp import Gmp
from gvmtools.helper import error_and_exit

HELP_TEXT = (
    "This script pulls targetname, hostnames/IP addresses, and credentials "
    "from a csv file and creates a target for each row. \n\n"
    "csv file to contain name of target, ips, and up to 4 credentials previously created \n"
    "name,ip-addresses,credential1,credential2,credential3,credential4"
)


def check_args(args):
    len_args = len(args.script) - 1
    if len_args != 1:
        message = """
        This script pulls target names from a csv file and creates a target \
for each row.
        One parameter after the script name is required.

        1. <targets_csvfile>  -- text file containing Targetname and hostnames or IP-addresses

        Example:
            $ gvm-script --gmp-username name --gmp-password pass \
ssh --hostname <gsm> scripts/create_targets_from_csv.gmp <targets_csvfile>
        """
        print(message)
        sys.exit()


def parse_args(args: Namespace) -> Namespace:  # pylint: disable=unused-argument
    """Parsing args ..."""

    parser = ArgumentParser(
        prefix_chars="+",
        add_help=False,
        formatter_class=RawTextHelpFormatter,
        description=HELP_TEXT,
    )

    parser.add_argument(
        "+h",
        "++help",
        action="help",
        help="Show this help message and exit.",
    )

    parser.add_argument(
        "targets_csv_file",
        type=str,
        help=("File containing host names / IPs"),
    )

    ports = parser.add_mutually_exclusive_group()
    ports.add_argument(
        "+pl",
        "++port-list-id",
        type=str,
        dest="port_list_id",
        help="UUID of existing port list.",
    )
    ports.add_argument(
        "+pr",
        "++port-range",
        dest="port_range",
        type=str,
        help=(
            "Port range to create port list from, e.g. "
            "T:1-1234 for ports 1-1234/TCP"
        ),
    )

    ports.set_defaults(
        port_list_id="730ef368-57e2-11e1-a90f-406186ea4fc5"
    )  # All TCP and Nmap top 100 UDP
    # Default portlists see also script list-portlists.gmp.py
    # | Name                          | ID
    # - | ----------------------------- | ------------------------------------
    # 1 | All IANA assigned TCP         | 33d0cd82-57c6-11e1-8ed1-406186ea4fc5
    # 2 | All IANA assigned TCP and UDP | 4a4717fe-57d2-11e1-9a26-406186ea4fc5
    # 3 | All TCP and Nmap top 100 UDP  | 730ef368-57e2-11e1-a90f-406186ea4fc5

    script_args, _ = parser.parse_known_args(args)
    return script_args


def credential_id(
    gmp: Gmp,
    credName: str,
):
    response_xml = gmp.get_credentials(
        filter_string="rows=-1, name=" + credName
    )
    credentials_xml = response_xml.xpath("credential")
    cred_id = ""

    for credential in credentials_xml:
        cred_id = credential.get("id")
    return cred_id


def target_id(
    gmp: Gmp,
    targetName: str,
):
    response_xml = gmp.get_targets(filter_string="rows=-1, name=" + targetName)
    targets_xml = response_xml.xpath("target")
    target_id = ""

    for target in targets_xml:
        target_id = target.get("id")
    return target_id


def create_targets(
    gmp: Gmp,
    target_csv_file: Path,
    port_list_id: str,
):
    try:
        numberTargets = 0
        with open(target_csv_file, encoding="utf-8") as csvFile:
            content = csv.reader(csvFile, delimiter=",")  # read the data
            for row in content:  # loop through each row
                if len(row) == 0:
                    continue
                name = row[0]
                hosts = [row[1]]
                smbCred = credential_id(gmp, row[2])
                sshCred = credential_id(gmp, row[3])
                aliveTest = row[6]
                if not aliveTest:
                    aliveTest = "Scan Config Default"
                alive_test = gmp.types.AliveTest((aliveTest))
                comment = f"Created: {time.strftime('%Y/%m/%d-%H:%M:%S')}"
                try:
                    if target_id(gmp, name):
                        print(f"Target: {name} exist, not creating...")
                        continue

                    print("Creating target: " + name)
                    gmp.create_target(
                        name=name,
                        comment=comment,
                        hosts=hosts,
                        port_list_id=port_list_id,
                        smb_credential_id=smbCred,
                        ssh_credential_id=sshCred,
                        alive_test=alive_test,
                    )
                    numberTargets = numberTargets + 1
                except GvmResponseError as gvmerr:
                    print(f"{gvmerr=}, name: {name}")
                    pass
        csvFile.close()  # close the csv file
    except IOError as e:
        error_and_exit(f"Failed to read target_csv_file: {str(e)} (exit)")

    if len(row) == 0:
        error_and_exit("Host file is empty (exit)")

    return numberTargets


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable
    if args.script:
        args = args.script[1:]

    parsed_args = parse_args(args=args)
    # port_list_id="4a4717fe-57d2-11e1-9a26-406186ea4fc5"

    print("Creating targets.\n")

    numberTargets = create_targets(
        gmp, parsed_args.targets_csv_file, parsed_args.port_list_id
    )

    numberTargets = str(numberTargets)
    print("    [" + numberTargets + "] Target(s) created!\n")


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2024 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

# Run with gvm-script --gmp-username admin-user --gmp-password password socket create-alerts-from-csv.gmp.py alerts.csv
#
# Information on Variables to be used in alerts: https://docs.greenbone.net/GSM-Manual/gos-22.04/en/scanning.html#using-alerts
# Example script: https://forum.greenbone.net/t/working-example-of-creating-an-alert-using-script/7511/2

import csv
import sys
import time
from argparse import ArgumentParser, Namespace, RawTextHelpFormatter
from pathlib import Path

from gvm.errors import GvmResponseError
from gvm.protocols.gmp import Gmp
from gvmtools.helper import error_and_exit

HELP_TEXT = (
    "This script pulls alert information "
    "from a csv file and creates a alert for each row. \n"
    "use the same alert names when creating tasks! \n\n"
    "Use example alerts.csv as a template \n\n"
    "It should be rather self explanatory."
)


def check_args(args):
    len_args = len(args.script) - 1
    if len_args != 2:
        message = """
        This script pulls alerts from a csv file and creates a \
alert for each row in the csv file.
        One parameter after the script name is required.

        1. <alerts_csvfile>  -- csv file containing names and secrets required for scan alerts

        Example:
            $ gvm-script --gmp-username name --gmp-password pass \
ssh --hostname <gsm> scripts/create_alerts_from_csv.gmp.py \
<alerts-csvfile>
        """
        print(message)
        sys.exit()


def parse_args(args: Namespace) -> Namespace:  # pylint: disable=unused-argument
    """Parsing args ..."""

    parser = ArgumentParser(
        prefix_chars="+",
        add_help=False,
        formatter_class=RawTextHelpFormatter,
        description=HELP_TEXT,
    )

    parser.add_argument(
        "+h",
        "++help",
        action="help",
        help="Show this help message and exit.",
    )

    parser.add_argument(
        "alert_file",
        type=str,
        help=("CSV File containing alerts"),
    )
    script_args, _ = parser.parse_known_args(args)
    return script_args


def alert_id(
    gmp: Gmp,
    alert_name: str,
):
    response_xml = gmp.get_alerts(filter_string="rows=-1, name=" + alert_name)
    alerts_xml = response_xml.xpath("alert")
    alert_id = ""

    for alert in alerts_xml:
        "".join(alert.xpath("name/text()"))
        alert_id = alert.get("id")
    return alert_id


def credential_id(
    gmp: Gmp,
    credential_name: str,
):
    response_xml = gmp.get_credentials(
        filter_string="rows=-1, name=" + credential_name
    )
    credentials_xml = response_xml.xpath("credential")
    credential_id = ""

    for credential in credentials_xml:
        credential_id = credential.get("id")
    return credential_id


def report_format_id(
    gmp: Gmp,
    report_format_name: str,
):
    response_xml = gmp.get_report_formats(
        details=True, filter_string="rows=-1, name=" + report_format_name
    )
    report_formats_xml = response_xml.xpath("report_format")
    report_format_id = ""

    for report_format in report_formats_xml:
        report_format_id = report_format.get("id")
    return report_format_id


def event_list(string):
    event_list = list(string.split(" "))
    return event_list


def create_alerts(
    gmp: Gmp,
    alert_file: Path,
):
    try:
        numberalerts = 0
        with open(alert_file, encoding="utf-8") as csvFile:
            content = csv.reader(csvFile, delimiter=",")  # read the data
            for row in content:  # loop through each row
                if len(row) == 0:
                    continue
                alert_name = row[0]
                str_alert_type = row[1]
                strRow2 = row[2]
                strRow3 = row[3]
                strRow4 = row[4]
                strRow5 = row[5]
                strRow6 = row[6]
                report_format = report_format_id(gmp, row[7])
                event_data = row[8]

                comment = f"Created: {time.strftime('%Y/%m/%d-%H:%M:%S')}"
                alert_type = getattr(gmp.types.AlertMethod, str_alert_type)

                if alert_id(gmp, alert_name):
                    print(f"Alert: {alert_name} exist, not creating...")
                    continue

                if str_alert_type == "EMAIL":
                    sender_email = strRow2
                    recipient_email = strRow3
                    subject = strRow4
                    message = strRow5
                    notice_type = strRow6
                    try:
                        print("Creating alert: " + alert_name)
                        gmp.create_alert(
                            name=alert_name,
                            comment=comment,
                            event=gmp.types.AlertEvent.TASK_RUN_STATUS_CHANGED,
                            event_data={"status": event_data},
                            condition=gmp.types.AlertCondition.ALWAYS,
                            method=alert_type,
                            method_data={
                                "message": message,
                                "notice": notice_type,
                                "from_address": sender_email,
                                "subject": subject,
                                "notice_report_format": report_format,
                                "notice_attach_format": report_format,
                                "to_address": recipient_email,
                            },
                        )
                        numberalerts = numberalerts + 1
                    except GvmResponseError as gvmerr:
                        print(f"{gvmerr=}, name: {alert_name}")
                        pass
                else:
                    smb_credential = credential_id(gmp, strRow2)
                    smb_share_path = strRow3
                    smb_report_name = strRow4
                    smb_folder = strRow5
                    smb_file_path = smb_folder + "/" + smb_report_name

                    try:
                        print("Creating alert: " + alert_name)
                        gmp.create_alert(
                            name=alert_name,
                            comment=comment,
                            event=gmp.types.AlertEvent.TASK_RUN_STATUS_CHANGED,
                            event_data={"status": event_data},
                            condition=gmp.types.AlertCondition.ALWAYS,
                            method=alert_type,
                            method_data={
                                "smb_credential": smb_credential,
                                "smb_share_path": smb_share_path,
                                "smb_report_format": report_format,
                                "smb_file_path": smb_file_path,
                            },
                        )
                        numberalerts = numberalerts + 1
                    except GvmResponseError as gvmerr:
                        print(f"{gvmerr=}, name: {alert_name}")
                        pass
        csvFile.close()  # close the csv file

    except IOError as e:
        error_and_exit(f"Failed to read alert_file: {str(e)} (exit)")

    if len(row) == 0:
        error_and_exit("alerts file is empty (exit)")

    return numberalerts


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable
    if args.script:
        args = args.script[1:]

    parsed_args = parse_args(args=args)

    print("Creating alerts.\n")

    numberalerts = create_alerts(
        gmp,
        parsed_args.alert_file,
    )

    numberalerts = str(numberalerts)
    print("    [" + numberalerts + "] alert(s) created!\n")


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2024 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later

from argparse import Namespace

from gvm.protocols.gmp import Gmp
from gvmtools.helper import Table


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=unused-argument

    response_xml = gmp.get_port_lists(filter_string="rows=-1")
    portlists_xml = response_xml.xpath("port_list")

    heading = ["#", "Name", "Id", "Ports All", "Ports TCP", "Ports UDP"]

    rows = []
    numberRows = 0

    print("Listing portlists.\n")

    for portlist in portlists_xml:
        # Count number of reports
        numberRows = numberRows + 1
        # Cast/convert to text to show in list
        rowNumber = str(numberRows)

        name = "".join(portlist.xpath("name/text()"))
        port_list_id = portlist.get("id")
        port_all = "".join(portlist.xpath("port_count/all/text()"))
        port_tcp = "".join(portlist.xpath("port_count/tcp/text()"))
        port_udp = "".join(portlist.xpath("port_count/udp/text()"))

        rows.append(
            [rowNumber, name, port_list_id, port_all, port_tcp, port_udp]
        )

    print(Table(heading=heading, rows=rows))


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2024 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later

from argparse import Namespace

from gvm.protocols.gmp import Gmp
from gvmtools.helper import Table


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=unused-argument

    response_xml = gmp.get_credentials(filter_string="rows=-1")
    credentials_xml = response_xml.xpath("credential")

    heading = ["#", "Id", "Name", "Type", "Insecure use"]

    rows = []
    numberRows = 0

    print("Listing credentials.\n")

    for credential in credentials_xml:
        # Count number of reports
        numberRows = numberRows + 1
        # Cast/convert to text to show in list
        rowNumber = str(numberRows)

        name = "".join(credential.xpath("name/text()"))
        credential_id = credential.get("id")
        cred_type = "".join(credential.xpath("type/text()"))
        if cred_type.upper() == "UP":
            cred_type = "Username + Password (up)"
        elif cred_type.upper() == "USK":
            cred_type = "Username + SSH Key (usk)"
        elif cred_type.upper() == "SMIME":
            cred_type = "S/MIME Certificate (smime)"
        elif cred_type.upper() == "PGP":
            cred_type = "PGP Encryption Key (pgp)"
        elif cred_type.upper() == "SNMP":
            cred_type = "Simple Network Management Protocol (snmp)"
        elif cred_type.upper() == "PW":
            cred_type = "Password only (pw)"
        cred_insecureuse = "".join(credential.xpath("allow_insecure/text()"))
        if cred_insecureuse == "1":
            cred_insecureuse = "Yes"
        else:
            cred_insecureuse = "No"

        rows.append(
            [rowNumber, credential_id, name, cred_type, cred_insecureuse]
        )

    print(Table(heading=heading, rows=rows))


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2017-2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import sys
from argparse import Namespace

from gvm.errors import GvmError
from gvm.protocols.gmp import Gmp


def check_args(args):
    len_args = len(args.script) - 1
    if len_args != 1:
        message = """
        This script creates a new scan config with nvts from a given CERT-Bund!
        It needs one parameter after the script name.

        1. <cert>   -- Name or ID of the CERT-Bund

        Example:
            $ gvm-script --gmp-username name --gmp-password pass \
    ssh --hostname <gsm> scripts/cfg-gen-for-certs.gmp.py CB-K16/0943
        """
        print(message)
        sys.exit()


def create_scan_config(gmp, cert_bund_name):
    cert_bund_details = gmp.get_info(
        info_id=cert_bund_name, info_type=gmp.types.InfoType.CERT_BUND_ADV
    )

    list_cves = cert_bund_details.xpath(
        "info/cert_bund_adv/raw_data/Advisory/CVEList/CVE/text()"
    )

    nvt_dict = dict()
    counter = 0

    for cve in list_cves:
        # Get all nvts of this cve
        cve_info = gmp.get_info(info_id=cve, info_type=gmp.types.InfoType.CVE)
        nvts = cve_info.xpath("info/cve/nvts/nvt")

        for nvt in nvts:
            counter += 1
            oid = nvt.xpath("@oid")[0]

            # We need the nvt family to modify scan config
            nvt_data = gmp.get_scan_config_nvt(oid)
            family = nvt_data.xpath("nvt/family/text()")[0]

            # Create key value map
            if family in nvt_dict and oid not in nvt_dict[family]:
                nvt_dict[family].append(oid)
            else:
                nvt_dict[family] = [oid]

    # Create new config
    copy_id = "085569ce-73ed-11df-83c3-002264764cea"
    config_name = f"scanconfig_for_{cert_bund_name}"
    config_id = ""

    try:
        res = gmp.create_scan_config(copy_id, config_name)
        config_id = res.xpath("@id")[0]

        # Modify the config with the nvts oid
        for family, nvt_oid in nvt_dict.items():
            try:
                gmp.modify_scan_config_set_nvt_selection(
                    config_id=config_id, nvt_oids=nvt_oid, family=family
                )
            except GvmError as gvmerr:
                print(f"{gvmerr=}")

        # This nvts must be present to work
        family = "Port scanners"
        nvts = ["1.3.6.1.4.1.25623.1.0.14259", "1.3.6.1.4.1.25623.1.0.100315"]
        gmp.modify_scan_config_set_nvt_selection(
            config_id=config_id, nvt_oids=nvts, family=family
        )

    except GvmError:
        print("Config exist")


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable

    check_args(args)

    cert_bund_name = args.script[1]

    print(f"Creating scan config for {cert_bund_name}")

    create_scan_config(gmp, cert_bund_name)


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2017-2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import csv
import sys
from argparse import Namespace

from gvm.protocols.gmp import Gmp


def check_args(args):
    len_args = len(args.script) - 1
    if len_args != 1:
        message = """
        This script reads host data from a csv file and sync it with the gsm.
        It needs one parameters after the script name.

        1. <csv_file> - should contain a table of IP-addresses with an
                        optional a comment

        Example:
            $ gvm-script --gmp-username name --gmp-password pass \
    ssh --hostname <gsm> scripts/sync-hosts.gmp.py <csv_file>
        """
        print(message)
        sys.exit()


def sync_hosts(gmp, filename):
    with open(filename, newline="", encoding="utf-8") as f:
        reader = csv.reader(f, delimiter=",", quotechar="|")
        for row in reader:
            if len(row) == 2:
                ip = row[0]
                comment = row[1]

                # check if host exists
                ret = gmp.get_hosts(filter_string=f"ip={ip}")
                if ret.xpath("host"):
                    print(f"\nAsset with IP {ip} exist")
                    host_id = ret.xpath("host/@id")[0]
                    gmp.delete_host(host_id=host_id)
                else:
                    print(f"Asset with ip {ip} does not exist. Sync...")
                    ret = gmp.create_host(name=ip, comment=comment)

                    if "OK" in ret.xpath("@status_text")[0]:
                        print("Asset synced")


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable

    check_args(args)

    file = args.script[1]

    sync_hosts(gmp, file)


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2017-2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import sys
from argparse import Namespace
from random import choice

from gvm.protocols.gmp import Gmp
from gvmtools.helper import generate_id


def check_args(args):
    len_args = len(args.script) - 1
    if len_args != 1:
        message = """
        This script will create random data in the given GVM database

        1. <count>  -- Number of datasets to create

        Example:
            $ gvm-script --gmp-username name --gmp-password pass \
    ssh --hostname <gsm> scripts/create-dummy-data.gmp.py <count>
        """
        print(message)
        sys.exit()


def create_data(gmp, count):
    config_ids = []
    target_ids = []

    for _ in range(0, count):
        name = generate_id()
        gmp.create_credential(
            name,
            login=name,
            password=name,
            credential_type=gmp.types.CredentialType.PASSWORD_ONLY,
        )
    print(str(count) + " random credentials generated.")

    for _ in range(0, count):
        name = generate_id()
        gmp.create_port_list(name, port_range="T:1-42")
    print(str(count) + " random port lists generated.")

    for _ in range(0, count):
        name = generate_id()
        res = gmp.create_scan_config(
            "085569ce-73ed-11df-83c3-002264764cea", name
        )
        config_ids.append(res.xpath("@id")[0])
    print(str(count) + " random scan configs generated.")

    for _ in range(0, count):
        name = generate_id()
        res = gmp.create_target(name, hosts=["127.0.0.1"])

        target_ids.append(res.xpath("@id")[0])
    print(str(count) + " random targets generated.")

    for _ in range(0, count):
        name = generate_id()
        config_id = choice(config_ids)
        target_id = choice(target_ids)
        gmp.create_task(
            name, config_id, target_id, "08b69003-5fc2-4037-a479-93b440211c73"
        )
    print(str(count) + " random tasks generated.")


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable

    check_args(args)

    create_data(gmp, int(args.script[1]))


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2017-2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from argparse import Namespace

from gvm.errors import GvmResponseError
from gvm.protocols.gmp import Gmp


def clean_sensor(gmp: Gmp) -> None:
    tasks = gmp.get_tasks(
        filter_string="rows=-1 not status=Running and "
        "not status=Requested and not "
        "status=&quot;Stop Requested&quot;"
    )

    try:
        for task_id in tasks.xpath("task/@id"):
            print(f"Removing task {task_id} ... ")
            status_text = gmp.delete_task(task_id, ultimate=True).xpath(
                "@status_text"
            )[0]
            print(status_text)
    except GvmResponseError as gvmerr:
        print(f"{gvmerr=}")
        pass

    targets = gmp.get_targets(filter_string="rows=-1 not _owner=&quot;&quot;")
    try:
        for target_id in targets.xpath("target/@id"):
            print(f"Removing target {target_id} ... ")
            status_text = gmp.delete_target(target_id, ultimate=True).xpath(
                "@status_text"
            )[0]
            print(status_text)
    except GvmResponseError as gvmerr:
        print(f"{gvmerr=}")
        pass

    configs = gmp.get_scan_configs(
        filter_string="rows=-1 not _owner=&quot;&quot;"
    )
    try:
        for config_id in configs.xpath("config/@id"):
            print(f"Removing config {config_id} ... ")
            status_text = gmp.delete_scan_config(
                config_id, ultimate=True
            ).xpath("@status_text")[0]
            print(status_text)
    except GvmResponseError as gvmerr:
        print(f"{gvmerr=}")
        pass

    port_lists = gmp.get_port_lists(
        filter_string="rows=-1 not _owner=&quot;&quot;"
    )
    try:
        for port_list_id in port_lists.xpath("port_list/@id"):
            print(f"Removing port_list {port_list_id} ... ")
            status_text = gmp.delete_port_list(
                port_list_id, ultimate=True
            ).xpath("@status_text")[0]
            print(status_text)
    except GvmResponseError as gvmerr:
        print(f"{gvmerr=}")
        pass

    alerts = gmp.get_alerts(filter_string="rows=-1 not _owner=&quot;&quot;")
    try:
        for alert_id in alerts.xpath("alert/@id"):
            print(f"Removing alert {alert_id} ... ")
            status_text = gmp.delete_alert(alert_id, ultimate=True).xpath(
                "@status_text"
            )[0]
            print(status_text)
    except GvmResponseError as gvmerr:
        print(f"{gvmerr=}")
        pass

    schedules = gmp.get_schedules(
        filter_string="rows=-1 not _owner=&quot;&quot;"
    )
    try:
        for schedule_id in schedules.xpath("schedule/@id"):
            print(f"Removing schedule {schedule_id} ... ")
            status_text = gmp.delete_schedule(schedule_id, ultimate=True).xpath(
                "@status_text"
            )[0]
            print(status_text)
    except GvmResponseError as gvmerr:
        print(f"{gvmerr=}")
        pass

    tags = gmp.get_tags(filter_string="rows=-1 not _owner=&quot;&quot;")
    try:
        for tag_id in tags.xpath("tag/@id"):
            print(f"Removing tag {tag_id} ... ")
            status_text = gmp.delete_tag(tag_id, ultimate=True).xpath(
                "@status_text"
            )[0]
            print(status_text)
    except GvmResponseError as gvmerr:
        print(f"{gvmerr=}")
        pass

    filters = gmp.get_filters(filter_string="rows=-1 not _owner=&quot;&quot;")
    try:
        for filter_id in filters.xpath("filter/@id"):
            print(f"Removing filter {filter_id} ... ")
            status_text = gmp.delete_filter(filter_id, ultimate=True).xpath(
                "@status_text"
            )[0]
            print(status_text)
    except GvmResponseError as gvmerr:
        print(f"{gvmerr=}")
        pass

    credentials = gmp.get_credentials(
        filter_string="rows=-1 not _owner=&quot;&quot;"
    )
    try:
        for config_id in credentials.xpath("credential/@id"):
            print(f"Removing credential {config_id} ... ")
            status_text = gmp.delete_credential(config_id, ultimate=True).xpath(
                "@status_text"
            )[0]
            print(status_text)
    except GvmResponseError as gvmerr:
        print(f"{gvmerr=}")
        pass

    print("Emptying trash... ")
    try:
        status_text = gmp.empty_trashcan().xpath("@status_text")[0]
        print(status_text)
    except GvmResponseError as gvmerr:
        print(f"{gvmerr=}")
        pass


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=unused-argument

    print(
        "This script removes all resources from a sensor, except active tasks.\n"
    )

    clean_sensor(gmp)


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2024 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later

from argparse import Namespace

from gvm.protocols.gmp import Gmp
from gvmtools.helper import Table


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=unused-argument

    response_xml = gmp.get_tickets(filter_string="rows=-1")
    tickets_xml = response_xml.xpath("ticket")

    heading = ["#", "ID", "Name", "Host", "Task", "Status", "Note"]

    rows = []
    numberRows = 0

    print("Listing tickets.\n")

    for ticket in tickets_xml:
        # Count number of reports
        numberRows = numberRows + 1
        # Cast/convert to text to show in list
        rowNumber = str(numberRows)

        name = "".join(ticket.xpath("name/text()"))
        ticket_id = ticket.get("id")
        ticket_status = "".join(ticket.xpath("status/text()"))
        ticket_task = "".join(ticket.xpath("task/name/text()"))
        ticket_host = "".join(ticket.xpath("host/text()"))
        if ticket_status.upper() == "OPEN":
            ticket_note = "".join(ticket.xpath("open_note/text()"))
        elif ticket_status.upper() == "FIXED":
            ticket_note = "".join(ticket.xpath("fixed_note/text()"))
        elif ticket_status.upper() == "CLOSED":
            ticket_note = "".join(ticket.xpath("closed_note/text()"))

        rows.append(
            [
                rowNumber,
                ticket_id,
                name,
                ticket_host,
                ticket_task,
                ticket_status,
                ticket_note,
            ]
        )

    print(Table(heading=heading, rows=rows))


if __name__ == "__gmp__":
    main(gmp, args)
